@@ -2,6 +2,7 @@
 
 namespace Illuminate\Mail;
 
+use Closure;
 use Illuminate\Contracts\Events\Dispatcher;
 use Illuminate\Contracts\Mail\Mailable as MailableContract;
 use Illuminate\Contracts\Mail\Mailer as MailerContract;
@@ -12,15 +13,25 @@ use Illuminate\Contracts\Support\Htmlable;
 use Illuminate\Contracts\View\Factory;
 use Illuminate\Mail\Events\MessageSending;
 use Illuminate\Mail\Events\MessageSent;
+use Illuminate\Mail\Mailables\Address;
 use Illuminate\Support\HtmlString;
 use Illuminate\Support\Traits\Macroable;
 use InvalidArgumentException;
-use Swift_Mailer;
+use Symfony\Component\Mailer\Envelope;
+use Symfony\Component\Mailer\Transport\TransportInterface;
+use Symfony\Component\Mime\Email;
 
 class Mailer implements MailerContract, MailQueueContract
 {
     use Macroable;
 
+    /**
+     * The name that is configured for the mailer.
+     *
+     * @var string
+     */
+    protected $name;
+
     /**
      * The view factory instance.
      *
@@ -29,11 +40,11 @@ class Mailer implements MailerContract, MailQueueContract
     protected $views;
 
     /**
-     * The Swift Mailer instance.
+     * The Symfony Transport instance.
      *
-     * @var \Swift_Mailer
+     * @var \Symfony\Component\Mailer\Transport\TransportInterface
      */
-    protected $swift;
+    protected $transport;
 
     /**
      * The event dispatcher instance.
@@ -56,6 +67,13 @@ class Mailer implements MailerContract, MailQueueContract
      */
     protected $replyTo;
 
+    /**
+     * The global return path address.
+     *
+     * @var array
+     */
+    protected $returnPath;
+
     /**
      * The global to address and name.
      *
@@ -70,26 +88,21 @@ class Mailer implements MailerContract, MailQueueContract
      */
     protected $queue;
 
-    /**
-     * Array of failed recipients.
-     *
-     * @var array
-     */
-    protected $failedRecipients = [];
-
     /**
      * Create a new Mailer instance.
      *
+     * @param  string  $name
      * @param  \Illuminate\Contracts\View\Factory  $views
-     * @param  \Swift_Mailer  $swift
+     * @param  \Symfony\Component\Mailer\Transport\TransportInterface  $transport
      * @param  \Illuminate\Contracts\Events\Dispatcher|null  $events
      * @return void
      */
-    public function __construct(Factory $views, Swift_Mailer $swift, Dispatcher $events = null)
+    public function __construct(string $name, Factory $views, TransportInterface $transport, ?Dispatcher $events = null)
     {
+        $this->name = $name;
         $this->views = $views;
-        $this->swift = $swift;
         $this->events = $events;
+        $this->transport = $transport;
     }
 
     /**
@@ -116,6 +129,17 @@ class Mailer implements MailerContract, MailQueueContract
         $this->replyTo = compact('address', 'name');
     }
 
+    /**
+     * Set the global return path address.
+     *
+     * @param  string  $address
+     * @return void
+     */
+    public function alwaysReturnPath($address)
+    {
+        $this->returnPath = compact('address');
+    }
+
     /**
      * Set the global to address and name.
      *
@@ -132,10 +156,15 @@ class Mailer implements MailerContract, MailQueueContract
      * Begin the process of mailing a mailable class instance.
      *
      * @param  mixed  $users
+     * @param  string|null  $name
      * @return \Illuminate\Mail\PendingMail
      */
-    public function to($users)
+    public function to($users, $name = null)
     {
+        if (! is_null($name) && is_string($users)) {
+            $users = new Address($users, $name);
+        }
+
         return (new PendingMail($this))->to($users);
     }
 
@@ -143,10 +172,15 @@ class Mailer implements MailerContract, MailQueueContract
      * Begin the process of mailing a mailable class instance.
      *
      * @param  mixed  $users
+     * @param  string|null  $name
      * @return \Illuminate\Mail\PendingMail
      */
-    public function cc($users)
+    public function cc($users, $name = null)
     {
+        if (! is_null($name) && is_string($users)) {
+            $users = new Address($users, $name);
+        }
+
         return (new PendingMail($this))->cc($users);
     }
 
@@ -154,10 +188,15 @@ class Mailer implements MailerContract, MailQueueContract
      * Begin the process of mailing a mailable class instance.
      *
      * @param  mixed  $users
+     * @param  string|null  $name
      * @return \Illuminate\Mail\PendingMail
      */
-    public function bcc($users)
+    public function bcc($users, $name = null)
     {
+        if (! is_null($name) && is_string($users)) {
+            $users = new Address($users, $name);
+        }
+
         return (new PendingMail($this))->bcc($users);
     }
 
@@ -166,7 +205,7 @@ class Mailer implements MailerContract, MailQueueContract
      *
      * @param  string  $html
      * @param  mixed  $callback
-     * @return void
+     * @return \Illuminate\Mail\SentMessage|null
      */
     public function html($html, $callback)
     {
@@ -178,7 +217,7 @@ class Mailer implements MailerContract, MailQueueContract
      *
      * @param  string  $text
      * @param  mixed  $callback
-     * @return void
+     * @return \Illuminate\Mail\SentMessage|null
      */
     public function raw($text, $callback)
     {
@@ -191,7 +230,7 @@ class Mailer implements MailerContract, MailQueueContract
      * @param  string  $view
      * @param  array  $data
      * @param  mixed  $callback
-     * @return void
+     * @return \Illuminate\Mail\SentMessage|null
      */
     public function plain($view, array $data, $callback)
     {
@@ -214,7 +253,38 @@ class Mailer implements MailerContract, MailQueueContract
 
         $data['message'] = $this->createMessage();
 
-        return $this->renderView($view ?: $plain, $data);
+        return $this->replaceEmbeddedAttachments(
+            $this->renderView($view ?: $plain, $data),
+            $data['message']->getSymfonyMessage()->getAttachments()
+        );
+    }
+
+    /**
+     * Replace the embedded image attachments with raw, inline image data for browser rendering.
+     *
+     * @param  string  $renderedView
+     * @param  array  $attachments
+     * @return string
+     */
+    protected function replaceEmbeddedAttachments(string $renderedView, array $attachments)
+    {
+        if (preg_match_all('/<img.+?src=[\'"]cid:([^\'"]+)[\'"].*?>/i', $renderedView, $matches)) {
+            foreach (array_unique($matches[1]) as $image) {
+                foreach ($attachments as $attachment) {
+                    if ($attachment->getFilename() === $image) {
+                        $renderedView = str_replace(
+                            'cid:'.$image,
+                            'data:'.$attachment->getContentType().';base64,'.$attachment->bodyToString(),
+                            $renderedView
+                        );
+
+                        break;
+                    }
+                }
+            }
+        }
+
+        return $renderedView;
     }
 
     /**
@@ -223,7 +293,7 @@ class Mailer implements MailerContract, MailQueueContract
      * @param  \Illuminate\Contracts\Mail\Mailable|string|array  $view
      * @param  array  $data
      * @param  \Closure|string|null  $callback
-     * @return void
+     * @return \Illuminate\Mail\SentMessage|null
      */
     public function send($view, array $data = [], $callback = null)
     {
@@ -231,6 +301,8 @@ class Mailer implements MailerContract, MailQueueContract
             return $this->sendMailable($view);
         }
 
+        $data['mailer'] = $this->name;
+
         // First we need to parse the view, which could either be a string or an array
         // containing both an HTML and plain text versions of the view which should
         // be used when sending an e-mail. We will extract both of them out here.
@@ -241,7 +313,9 @@ class Mailer implements MailerContract, MailQueueContract
         // Once we have retrieved the view content for the e-mail we will set the body
         // of this message using the HTML type, which will provide a simple wrapper
         // to creating view based emails that are able to receive arrays of data.
-        $callback($message);
+        if (! is_null($callback)) {
+            $callback($message);
+        }
 
         $this->addContent($message, $view, $plain, $raw, $data);
 
@@ -255,12 +329,18 @@ class Mailer implements MailerContract, MailQueueContract
         // Next we will determine if the message should be sent. We give the developer
         // one final chance to stop this message and then we will send it to all of
         // its recipients. We will then fire the sent event for the sent message.
-        $swiftMessage = $message->getSwiftMessage();
+        $symfonyMessage = $message->getSymfonyMessage();
 
-        if ($this->shouldSendMessage($swiftMessage, $data)) {
-            $this->sendSwiftMessage($swiftMessage);
+        if ($this->shouldSendMessage($symfonyMessage, $data)) {
+            $symfonySentMessage = $this->sendSymfonyMessage($symfonyMessage);
 
-            $this->dispatchSentEvent($message, $data);
+            if ($symfonySentMessage) {
+                $sentMessage = new SentMessage($symfonySentMessage);
+
+                $this->dispatchSentEvent($sentMessage, $data);
+
+                return $sentMessage;
+            }
         }
     }
 
@@ -268,26 +348,26 @@ class Mailer implements MailerContract, MailQueueContract
      * Send the given mailable.
      *
      * @param  \Illuminate\Contracts\Mail\Mailable  $mailable
-     * @return mixed
+     * @return \Illuminate\Mail\SentMessage|null
      */
     protected function sendMailable(MailableContract $mailable)
     {
         return $mailable instanceof ShouldQueue
-                        ? $mailable->queue($this->queue)
-                        : $mailable->send($this);
+                        ? $mailable->mailer($this->name)->queue($this->queue)
+                        : $mailable->mailer($this->name)->send($this);
     }
 
     /**
      * Parse the given view name or array.
      *
-     * @param  string|array  $view
+     * @param  \Closure|array|string  $view
      * @return array
      *
      * @throws \InvalidArgumentException
      */
     protected function parseView($view)
     {
-        if (is_string($view)) {
+        if (is_string($view) || $view instanceof Closure) {
             return [$view, null, null];
         }
 
@@ -325,31 +405,29 @@ class Mailer implements MailerContract, MailQueueContract
     protected function addContent($message, $view, $plain, $raw, $data)
     {
         if (isset($view)) {
-            $message->setBody($this->renderView($view, $data) ?: ' ', 'text/html');
+            $message->html($this->renderView($view, $data) ?: ' ');
         }
 
         if (isset($plain)) {
-            $method = isset($view) ? 'addPart' : 'setBody';
-
-            $message->$method($this->renderView($plain, $data) ?: ' ', 'text/plain');
+            $message->text($this->renderView($plain, $data) ?: ' ');
         }
 
         if (isset($raw)) {
-            $method = (isset($view) || isset($plain)) ? 'addPart' : 'setBody';
-
-            $message->$method($raw, 'text/plain');
+            $message->text($raw);
         }
     }
 
     /**
      * Render the given view.
      *
-     * @param  string  $view
+     * @param  \Closure|string  $view
      * @param  array  $data
      * @return string
      */
     protected function renderView($view, $data)
     {
+        $view = value($view, $data);
+
         return $view instanceof Htmlable
                         ? $view->toHtml()
                         : $this->views->make($view, $data)->render();
@@ -363,9 +441,12 @@ class Mailer implements MailerContract, MailQueueContract
      */
     protected function setGlobalToAndRemoveCcAndBcc($message)
     {
+        $message->forgetTo();
+
         $message->to($this->to['address'], $this->to['name'], true);
-        $message->cc(null, null, true);
-        $message->bcc(null, null, true);
+
+        $message->forgetCc();
+        $message->forgetBcc();
     }
 
     /**
@@ -387,7 +468,7 @@ class Mailer implements MailerContract, MailQueueContract
             $view->onQueue($queue);
         }
 
-        return $view->queue($this->queue);
+        return $view->mailer($this->name)->queue($this->queue);
     }
 
     /**
@@ -432,7 +513,9 @@ class Mailer implements MailerContract, MailQueueContract
             throw new InvalidArgumentException('Only mailables may be queued.');
         }
 
-        return $view->later($delay, is_null($queue) ? $this->queue : $queue);
+        return $view->mailer($this->name)->later(
+            $delay, is_null($queue) ? $this->queue : $queue
+        );
     }
 
     /**
@@ -455,7 +538,7 @@ class Mailer implements MailerContract, MailQueueContract
      */
     protected function createMessage()
     {
-        $message = new Message($this->swift->createMessage('message'));
+        $message = new Message(new Email());
 
         // If a global from address has been specified we will set it on every message
         // instance so the developer does not have to repeat themselves every time
@@ -471,30 +554,32 @@ class Mailer implements MailerContract, MailQueueContract
             $message->replyTo($this->replyTo['address'], $this->replyTo['name']);
         }
 
+        if (! empty($this->returnPath['address'])) {
+            $message->returnPath($this->returnPath['address']);
+        }
+
         return $message;
     }
 
     /**
-     * Send a Swift Message instance.
+     * Send a Symfony Email instance.
      *
-     * @param  \Swift_Message  $message
-     * @return int|null
+     * @param  \Symfony\Component\Mime\Email  $message
+     * @return \Symfony\Component\Mailer\SentMessage|null
      */
-    protected function sendSwiftMessage($message)
+    protected function sendSymfonyMessage(Email $message)
     {
-        $this->failedRecipients = [];
-
         try {
-            return $this->swift->send($message, $this->failedRecipients);
+            return $this->transport->send($message, Envelope::create($message));
         } finally {
-            $this->forceReconnection();
+            //
         }
     }
 
     /**
-     * Determines if the message can be sent.
+     * Determines if the email can be sent.
      *
-     * @param  \Swift_Message  $message
+     * @param  \Symfony\Component\Mime\Email  $message
      * @param  array  $data
      * @return bool
      */
@@ -512,7 +597,7 @@ class Mailer implements MailerContract, MailQueueContract
     /**
      * Dispatch the message sent event.
      *
-     * @param  \Illuminate\Mail\Message  $message
+     * @param  \Illuminate\Mail\SentMessage  $message
      * @param  array  $data
      * @return void
      */
@@ -520,41 +605,19 @@ class Mailer implements MailerContract, MailQueueContract
     {
         if ($this->events) {
             $this->events->dispatch(
-                new MessageSent($message->getSwiftMessage(), $data)
+                new MessageSent($message, $data)
             );
         }
     }
 
     /**
-     * Force the transport to re-connect.
+     * Get the Symfony Transport instance.
      *
-     * This will prevent errors in daemon queue situations.
-     *
-     * @return void
+     * @return \Symfony\Component\Mailer\Transport\TransportInterface
      */
-    protected function forceReconnection()
+    public function getSymfonyTransport()
     {
-        $this->getSwiftMailer()->getTransport()->stop();
-    }
-
-    /**
-     * Get the array of failed recipients.
-     *
-     * @return array
-     */
-    public function failures()
-    {
-        return $this->failedRecipients;
-    }
-
-    /**
-     * Get the Swift Mailer instance.
-     *
-     * @return \Swift_Mailer
-     */
-    public function getSwiftMailer()
-    {
-        return $this->swift;
+        return $this->transport;
     }
 
     /**
@@ -568,14 +631,14 @@ class Mailer implements MailerContract, MailQueueContract
     }
 
     /**
-     * Set the Swift Mailer instance.
+     * Set the Symfony Transport instance.
      *
-     * @param  \Swift_Mailer  $swift
+     * @param  \Symfony\Component\Mailer\Transport\TransportInterface  $transport
      * @return void
      */
-    public function setSwiftMailer($swift)
+    public function setSymfonyTransport(TransportInterface $transport)
     {
-        $this->swift = $swift;
+        $this->transport = $transport;
     }
 
     /**
