@@ -98,12 +98,12 @@ abstract class Manager
         // callbacks allow developers to build their own "drivers" easily using Closures.
         if (isset($this->customCreators[$driver])) {
             return $this->callCustomCreator($driver);
-        } else {
-            $method = 'create'.Str::studly($driver).'Driver';
+        }
 
-            if (method_exists($this, $method)) {
-                return $this->$method();
-            }
+        $method = 'create'.Str::studly($driver).'Driver';
+
+        if (method_exists($this, $method)) {
+            return $this->$method();
         }
 
         throw new InvalidArgumentException("Driver [$driver] not supported.");
