@@ -6,6 +6,7 @@ use Illuminate\Auth\AuthenticationException;
 use Illuminate\Auth\AuthManager;
 use Illuminate\Auth\EloquentUserProvider;
 use Illuminate\Auth\Middleware\Authenticate;
+use Illuminate\Auth\Middleware\AuthenticateWithBasicAuth;
 use Illuminate\Auth\RequestGuard;
 use Illuminate\Config\Repository as Config;
 use Illuminate\Container\Container;
@@ -36,6 +37,30 @@ class AuthenticateMiddlewareTest extends TestCase
         Container::setInstance(null);
     }
 
+    public function testItCanGenerateDefinitionViaStaticMethod()
+    {
+        $signature = (string) Authenticate::using('foo');
+        $this->assertSame('Illuminate\Auth\Middleware\Authenticate:foo', $signature);
+
+        $signature = (string) Authenticate::using('foo', 'bar');
+        $this->assertSame('Illuminate\Auth\Middleware\Authenticate:foo,bar', $signature);
+
+        $signature = (string) Authenticate::using('foo', 'bar', 'baz');
+        $this->assertSame('Illuminate\Auth\Middleware\Authenticate:foo,bar,baz', $signature);
+    }
+
+    public function testItCanGenerateDefinitionViaStaticMethodForBasic()
+    {
+        $signature = (string) AuthenticateWithBasicAuth::using('guard');
+        $this->assertSame('Illuminate\Auth\Middleware\AuthenticateWithBasicAuth:guard', $signature);
+
+        $signature = (string) AuthenticateWithBasicAuth::using('guard', 'field');
+        $this->assertSame('Illuminate\Auth\Middleware\AuthenticateWithBasicAuth:guard,field', $signature);
+
+        $signature = (string) AuthenticateWithBasicAuth::using(field: 'field');
+        $this->assertSame('Illuminate\Auth\Middleware\AuthenticateWithBasicAuth:,field', $signature);
+    }
+
     public function testDefaultUnauthenticatedThrows()
     {
         $this->expectException(AuthenticationException::class);
