@@ -7,7 +7,6 @@ use Illuminate\Support\Carbon;
 use Memcached;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
-use stdClass;
 
 /**
  * @requires extension memcached
@@ -23,7 +22,7 @@ class CacheMemcachedStoreTest extends TestCase
 
     public function testGetReturnsNullWhenNotFound()
     {
-        $memcache = $this->getMockBuilder(stdClass::class)->addMethods(['get', 'getResultCode'])->getMock();
+        $memcache = $this->getMockBuilder(Memcached::class)->onlyMethods(['get', 'getResultCode'])->getMock();
         $memcache->expects($this->once())->method('get')->with($this->equalTo('foo:bar'))->willReturn(null);
         $memcache->expects($this->once())->method('getResultCode')->willReturn(1);
         $store = new MemcachedStore($memcache, 'foo');
@@ -32,7 +31,7 @@ class CacheMemcachedStoreTest extends TestCase
 
     public function testMemcacheValueIsReturned()
     {
-        $memcache = $this->getMockBuilder(stdClass::class)->addMethods(['get', 'getResultCode'])->getMock();
+        $memcache = $this->getMockBuilder(Memcached::class)->onlyMethods(['get', 'getResultCode'])->getMock();
         $memcache->expects($this->once())->method('get')->willReturn('bar');
         $memcache->expects($this->once())->method('getResultCode')->willReturn(0);
         $store = new MemcachedStore($memcache);
@@ -41,7 +40,7 @@ class CacheMemcachedStoreTest extends TestCase
 
     public function testMemcacheGetMultiValuesAreReturnedWithCorrectKeys()
     {
-        $memcache = $this->getMockBuilder(stdClass::class)->addMethods(['getMulti', 'getResultCode'])->getMock();
+        $memcache = $this->getMockBuilder(Memcached::class)->onlyMethods(['getMulti', 'getResultCode'])->getMock();
         $memcache->expects($this->once())->method('getMulti')->with(
             ['foo:foo', 'foo:bar', 'foo:baz']
         )->willReturn([
@@ -66,16 +65,11 @@ class CacheMemcachedStoreTest extends TestCase
         $store = new MemcachedStore($memcache);
         $result = $store->put('foo', 'bar', 60);
         $this->assertTrue($result);
-        Carbon::setTestNow();
+        Carbon::setTestNow(null);
     }
 
     public function testIncrementMethodProperlyCallsMemcache()
     {
-        /* @link https://github.com/php-memcached-dev/php-memcached/pull/468 */
-        if (version_compare(phpversion(), '8.0.0', '>=')) {
-            $this->markTestSkipped('Test broken due to parse error in PHP Memcached.');
-        }
-
         $memcached = m::mock(Memcached::class);
         $memcached->shouldReceive('increment')->with('foo', 5)->once()->andReturn(5);
 
@@ -85,11 +79,6 @@ class CacheMemcachedStoreTest extends TestCase
 
     public function testDecrementMethodProperlyCallsMemcache()
     {
-        /* @link https://github.com/php-memcached-dev/php-memcached/pull/468 */
-        if (version_compare(phpversion(), '8.0.0', '>=')) {
-            $this->markTestSkipped('Test broken due to parse error in PHP Memcached.');
-        }
-
         $memcached = m::mock(Memcached::class);
         $memcached->shouldReceive('decrement')->with('foo', 5)->once()->andReturn(0);
 
