@@ -6,6 +6,7 @@ use Illuminate\Console\Application;
 use Illuminate\Console\Command;
 use Illuminate\Contracts\Events\Dispatcher;
 use Illuminate\Contracts\Foundation\Application as ApplicationContract;
+use Illuminate\Tests\Console\Fixtures\FakeCommandWithInputPrompting;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
 use Symfony\Component\Console\Command\Command as SymfonyCommand;
@@ -77,6 +78,42 @@ class ConsoleApplicationTest extends TestCase
         $this->assertSame($outputOfCallingArrayInput, $outputOfCallingStringInput);
     }
 
+    public function testCommandInputPromptsWhenRequiredArgumentIsMissing()
+    {
+        $app = new Application(
+            $laravel = new \Illuminate\Foundation\Application(__DIR__),
+            $events = m::mock(Dispatcher::class, ['dispatch' => null, 'fire' => null]),
+            'testing'
+        );
+
+        $app->addCommands([$command = new FakeCommandWithInputPrompting()]);
+
+        $command->setLaravel($laravel);
+
+        $statusCode = $app->call('fake-command-for-testing');
+
+        $this->assertTrue($command->prompted);
+        $this->assertSame(0, $statusCode);
+    }
+
+    public function testCommandInputDoesntPromptWhenRequiredArgumentIsPassed()
+    {
+        $app = new Application(
+            $app = new \Illuminate\Foundation\Application(__DIR__),
+            $events = m::mock(Dispatcher::class, ['dispatch' => null, 'fire' => null]),
+            'testing'
+        );
+
+        $app->addCommands([$command = new FakeCommandWithInputPrompting()]);
+
+        $statusCode = $app->call('fake-command-for-testing', [
+            'name' => 'foo',
+        ]);
+
+        $this->assertFalse($command->prompted);
+        $this->assertSame(0, $statusCode);
+    }
+
     protected function getMockConsole(array $methods)
     {
         $app = m::mock(ApplicationContract::class, ['version' => '6.0']);
