@@ -5,7 +5,6 @@ namespace Illuminate\Tests\Container;
 use Illuminate\Container\Container;
 use Illuminate\Container\EntryNotFoundException;
 use Illuminate\Contracts\Container\BindingResolutionException;
-use Illuminate\Contracts\Container\CircularDependencyException;
 use PHPUnit\Framework\TestCase;
 use Psr\Container\ContainerExceptionInterface;
 use stdClass;
@@ -117,6 +116,20 @@ class ContainerTest extends TestCase
         $this->assertSame($firstInstantiation, $secondInstantiation);
     }
 
+    public function testScopedIf()
+    {
+        $container = new Container;
+        $container->scopedIf('class', function () {
+            return 'foo';
+        });
+        $this->assertSame('foo', $container->make('class'));
+        $container->scopedIf('class', function () {
+            return 'bar';
+        });
+        $this->assertSame('foo', $container->make('class'));
+        $this->assertNotSame('bar', $container->make('class'));
+    }
+
     public function testScopedClosureResets()
     {
         $container = new Container;
@@ -200,13 +213,24 @@ class ContainerTest extends TestCase
     public function testArrayAccess()
     {
         $container = new Container;
+        $this->assertFalse(isset($container['something']));
         $container['something'] = function () {
             return 'foo';
         };
         $this->assertTrue(isset($container['something']));
+        $this->assertNotEmpty($container['something']);
         $this->assertSame('foo', $container['something']);
         unset($container['something']);
         $this->assertFalse(isset($container['something']));
+
+        //test offsetSet when it's not instanceof Closure
+        $container = new Container;
+        $container['something'] = 'text';
+        $this->assertTrue(isset($container['something']));
+        $this->assertNotEmpty($container['something']);
+        $this->assertSame('text', $container['something']);
+        unset($container['something']);
+        $this->assertFalse(isset($container['something']));
     }
 
     public function testAliases()
@@ -265,6 +289,21 @@ class ContainerTest extends TestCase
         $this->assertSame('taylor', $instance->default);
     }
 
+    public function testBound()
+    {
+        $container = new Container;
+        $container->bind(ContainerConcreteStub::class, function () {
+            //
+        });
+        $this->assertTrue($container->bound(ContainerConcreteStub::class));
+        $this->assertFalse($container->bound(IContainerContractStub::class));
+
+        $container = new Container;
+        $container->bind(IContainerContractStub::class, ContainerConcreteStub::class);
+        $this->assertTrue($container->bound(IContainerContractStub::class));
+        $this->assertFalse($container->bound(ContainerConcreteStub::class));
+    }
+
     public function testUnsetRemoveBoundInstances()
     {
         $container = new Container;
@@ -442,6 +481,18 @@ class ContainerTest extends TestCase
         $this->assertSame('ConcreteStub', $container->getAlias('foo'));
     }
 
+    public function testGetAliasRecursive()
+    {
+        $container = new Container;
+        $container->alias('ConcreteStub', 'foo');
+        $container->alias('foo', 'bar');
+        $container->alias('bar', 'baz');
+        $this->assertSame('ConcreteStub', $container->getAlias('baz'));
+        $this->assertTrue($container->isAlias('baz'));
+        $this->assertTrue($container->isAlias('bar'));
+        $this->assertTrue($container->isAlias('foo'));
+    }
+
     public function testItThrowsExceptionWhenAbstractIsSameAsAlias()
     {
         $this->expectException('LogicException');
@@ -494,6 +545,16 @@ class ContainerTest extends TestCase
         $this->assertEquals([1, 2, 3], $container->make('foo', [1, 2, 3]));
     }
 
+    public function testResolvingWithArrayOfMixedParameters()
+    {
+        $container = new Container;
+        $instance = $container->make(ContainerMixedPrimitiveStub::class, ['first' => 1, 'last' => 2, 'third' => 3]);
+        $this->assertSame(1, $instance->first);
+        $this->assertInstanceOf(ContainerConcreteStub::class, $instance->stub);
+        $this->assertSame(2, $instance->last);
+        $this->assertFalse(isset($instance->third));
+    }
+
     public function testResolvingWithUsingAnInterface()
     {
         $container = new Container;
@@ -604,9 +665,24 @@ class ContainerTest extends TestCase
         $this->assertInstanceOf(ContainerConcreteStub::class, $class);
     }
 
+    public function testMethodLevelContextualBinding()
+    {
+        $container = new Container;
+
+        $container->bind(IContainerContractStub::class, ContainerImplementationStubTwo::class);
+
+        $container->when(ContainerContextualBindingCallTarget::class)
+                ->needs(IContainerContractStub::class)
+                ->give(ContainerImplementationStub::class);
+
+        $result = $container->call([new ContainerContextualBindingCallTarget, 'work']);
+
+        $this->assertInstanceOf(ContainerImplementationStub::class, $result);
+    }
+
     // public function testContainerCanCatchCircularDependency()
     // {
-    //     $this->expectException(CircularDependencyException::class);
+    //     $this->expectException(\Illuminate\Contracts\Container\CircularDependencyException::class);
 
     //     $container = new Container;
     //     $container->get(CircularAStub::class);
@@ -722,3 +798,15 @@ class ContainerInjectVariableStubWithInterfaceImplementation implements IContain
         $this->something = $something;
     }
 }
+
+class ContainerContextualBindingCallTarget
+{
+    public function __construct()
+    {
+    }
+
+    public function work(IContainerContractStub $stub)
+    {
+        return $stub;
+    }
+}
