@@ -481,6 +481,28 @@ class ContextualBindingTest extends TestCase
         $this->assertSame('hunter42', $resolvedInstance->settings['password']);
         $this->assertSame('lumen', $resolvedInstance->settings['alias']);
     }
+
+    public function testContextualBindingWorksForMethodInvocation()
+    {
+        $container = new Container;
+
+        $container
+            ->when(ContainerTestContextInjectMethodArgument::class)
+            ->needs(IContainerContextContractStub::class)
+            ->give(ContainerContextImplementationStub::class);
+
+        $object = new ContainerTestContextInjectMethodArgument;
+
+        // array callable syntax...
+        $valueResolvedUsingArraySyntax = $container->call([$object, 'method']);
+        $this->assertInstanceOf(ContainerContextImplementationStub::class, $valueResolvedUsingArraySyntax);
+
+        if (PHP_VERSION_ID >= 80200) {
+            // first class callable syntax...
+            $valueResolvedUsingFirstClassSyntax = $container->call($object->method(...));
+            $this->assertInstanceOf(ContainerContextImplementationStub::class, $valueResolvedUsingFirstClassSyntax);
+        }
+    }
 }
 
 interface IContainerContextContractStub
@@ -559,7 +581,7 @@ class ContainerTestContextWithOptionalInnerDependency
 {
     public $inner;
 
-    public function __construct(ContainerTestContextInjectOne $inner = null)
+    public function __construct(?ContainerTestContextInjectOne $inner = null)
     {
         $this->inner = $inner;
     }
@@ -620,3 +642,11 @@ class ContainerTestContextInjectFromConfigArray
         $this->settings = $settings;
     }
 }
+
+class ContainerTestContextInjectMethodArgument
+{
+    public function method(IContainerContextContractStub $dependency)
+    {
+        return $dependency;
+    }
+}
