@@ -120,10 +120,10 @@ class CookieTest extends TestCase
         $cookieJar->expire('foobar', '/path', '/domain');
 
         $cookie = $cookieJar->queued('foobar');
-        $this->assertEquals('foobar', $cookie->getName());
+        $this->assertSame('foobar', $cookie->getName());
         $this->assertEquals(null, $cookie->getValue());
-        $this->assertEquals('/path', $cookie->getPath());
-        $this->assertEquals('/domain', $cookie->getDomain());
+        $this->assertSame('/path', $cookie->getPath());
+        $this->assertSame('/domain', $cookie->getDomain());
         $this->assertTrue($cookie->getExpiresTime() < time());
         $this->assertCount(1, $cookieJar->getQueuedCookies());
     }
@@ -217,7 +217,6 @@ class CookieTest extends TestCase
     private function getQueuedPropertyValue(CookieJar $cookieJar)
     {
         $property = (new ReflectionObject($cookieJar))->getProperty('queued');
-        $property->setAccessible(true);
 
         return $property->getValue($cookieJar);
     }
