@@ -45,7 +45,7 @@ class DatabaseConnectorTest extends TestCase
         $this->assertSame($result, $connection);
     }
 
-    public function mySqlConnectProvider()
+    public static function mySqlConnectProvider()
     {
         return [
             ['mysql:host=foo;dbname=bar', ['host' => 'foo', 'database' => 'bar', 'collation' => 'utf8_unicode_ci', 'charset' => 'utf8']],
@@ -75,49 +75,123 @@ class DatabaseConnectorTest extends TestCase
 
     public function testPostgresConnectCallsCreateConnectionWithProperArguments()
     {
-        $dsn = 'pgsql:host=foo;dbname=\'bar\';port=111';
+        $dsn = 'pgsql:host=foo;dbname=\'bar\';port=111;client_encoding=\'utf8\'';
         $config = ['host' => 'foo', 'database' => 'bar', 'port' => 111, 'charset' => 'utf8'];
         $connector = $this->getMockBuilder(PostgresConnector::class)->onlyMethods(['createConnection', 'getOptions'])->getMock();
         $connection = m::mock(stdClass::class);
         $connector->expects($this->once())->method('getOptions')->with($this->equalTo($config))->willReturn(['options']);
         $connector->expects($this->once())->method('createConnection')->with($this->equalTo($dsn), $this->equalTo($config), $this->equalTo(['options']))->willReturn($connection);
         $statement = m::mock(PDOStatement::class);
-        $connection->shouldReceive('prepare')->once()->with('set names \'utf8\'')->andReturn($statement);
+        $connection->shouldReceive('prepare')->zeroOrMoreTimes()->andReturn($statement);
+        $statement->shouldReceive('execute')->zeroOrMoreTimes();
+        $result = $connector->connect($config);
+
+        $this->assertSame($result, $connection);
+    }
+
+    /**
+     * @dataProvider provideSearchPaths
+     *
+     * @param  string  $searchPath
+     * @param  string  $expectedSql
+     */
+    public function testPostgresSearchPathIsSet($searchPath, $expectedSql)
+    {
+        $dsn = 'pgsql:host=foo;dbname=\'bar\';client_encoding=\'utf8\'';
+        $config = ['host' => 'foo', 'database' => 'bar', 'search_path' => $searchPath, 'charset' => 'utf8'];
+        $connector = $this->getMockBuilder(PostgresConnector::class)->onlyMethods(['createConnection', 'getOptions'])->getMock();
+        $connection = m::mock(stdClass::class);
+        $connector->expects($this->once())->method('getOptions')->with($this->equalTo($config))->willReturn(['options']);
+        $connector->expects($this->once())->method('createConnection')->with($this->equalTo($dsn), $this->equalTo($config), $this->equalTo(['options']))->willReturn($connection);
+        $statement = m::mock(PDOStatement::class);
+        $connection->shouldReceive('prepare')->once()->with($expectedSql)->andReturn($statement);
         $statement->shouldReceive('execute')->once();
         $result = $connector->connect($config);
 
         $this->assertSame($result, $connection);
     }
 
-    public function testPostgresSearchPathIsSet()
+    public static function provideSearchPaths()
     {
-        $dsn = 'pgsql:host=foo;dbname=\'bar\'';
-        $config = ['host' => 'foo', 'database' => 'bar', 'schema' => 'public', 'charset' => 'utf8'];
-        $connector = $this->getMockBuilder(PostgresConnector::class)->onlyMethods(['createConnection', 'getOptions'])->getMock();
-        $connection = m::mock(stdClass::class);
-        $connector->expects($this->once())->method('getOptions')->with($this->equalTo($config))->willReturn(['options']);
-        $connector->expects($this->once())->method('createConnection')->with($this->equalTo($dsn), $this->equalTo($config), $this->equalTo(['options']))->willReturn($connection);
-        $statement = m::mock(PDOStatement::class);
-        $connection->shouldReceive('prepare')->once()->with('set names \'utf8\'')->andReturn($statement);
-        $connection->shouldReceive('prepare')->once()->with('set search_path to "public"')->andReturn($statement);
-        $statement->shouldReceive('execute')->twice();
-        $result = $connector->connect($config);
-
-        $this->assertSame($result, $connection);
+        return [
+            'all-lowercase' => [
+                'public',
+                'set search_path to "public"',
+            ],
+            'case-sensitive' => [
+                'Public',
+                'set search_path to "Public"',
+            ],
+            'special characters' => [
+                '¡foo_bar-Baz!.Áüõß',
+                'set search_path to "¡foo_bar-Baz!.Áüõß"',
+            ],
+            'single-quoted' => [
+                "'public'",
+                'set search_path to "public"',
+            ],
+            'double-quoted' => [
+                '"public"',
+                'set search_path to "public"',
+            ],
+            'variable' => [
+                '$user',
+                'set search_path to "$user"',
+            ],
+            'delimit space' => [
+                'public user',
+                'set search_path to "public", "user"',
+            ],
+            'delimit newline' => [
+                "public\nuser\r\n\ttest",
+                'set search_path to "public", "user", "test"',
+            ],
+            'delimit comma' => [
+                'public,user',
+                'set search_path to "public", "user"',
+            ],
+            'delimit comma and space' => [
+                'public, user',
+                'set search_path to "public", "user"',
+            ],
+            'single-quoted many' => [
+                "'public', 'user'",
+                'set search_path to "public", "user"',
+            ],
+            'double-quoted many' => [
+                '"public", "user"',
+                'set search_path to "public", "user"',
+            ],
+            'quoted space is unsupported in string' => [
+                '"public user"',
+                'set search_path to "public", "user"',
+            ],
+            'array' => [
+                ['public', 'user'],
+                'set search_path to "public", "user"',
+            ],
+            'array with variable' => [
+                ['public', '$user'],
+                'set search_path to "public", "$user"',
+            ],
+            'array with delimiter characters' => [
+                ['public', '"user"', "'test'", 'spaced schema'],
+                'set search_path to "public", "user", "test", "spaced schema"',
+            ],
+        ];
     }
 
-    public function testPostgresSearchPathArraySupported()
+    public function testPostgresSearchPathFallbackToConfigKeySchema()
     {
-        $dsn = 'pgsql:host=foo;dbname=\'bar\'';
-        $config = ['host' => 'foo', 'database' => 'bar', 'schema' => ['public', 'user'], 'charset' => 'utf8'];
+        $dsn = 'pgsql:host=foo;dbname=\'bar\';client_encoding=\'utf8\'';
+        $config = ['host' => 'foo', 'database' => 'bar', 'schema' => ['public', '"user"'], 'charset' => 'utf8'];
         $connector = $this->getMockBuilder(PostgresConnector::class)->onlyMethods(['createConnection', 'getOptions'])->getMock();
         $connection = m::mock(stdClass::class);
         $connector->expects($this->once())->method('getOptions')->with($this->equalTo($config))->willReturn(['options']);
         $connector->expects($this->once())->method('createConnection')->with($this->equalTo($dsn), $this->equalTo($config), $this->equalTo(['options']))->willReturn($connection);
         $statement = m::mock(PDOStatement::class);
-        $connection->shouldReceive('prepare')->once()->with('set names \'utf8\'')->andReturn($statement);
         $connection->shouldReceive('prepare')->once()->with('set search_path to "public", "user"')->andReturn($statement);
-        $statement->shouldReceive('execute')->twice();
+        $statement->shouldReceive('execute')->once();
         $result = $connector->connect($config);
 
         $this->assertSame($result, $connection);
@@ -125,16 +199,47 @@ class DatabaseConnectorTest extends TestCase
 
     public function testPostgresApplicationNameIsSet()
     {
-        $dsn = 'pgsql:host=foo;dbname=\'bar\'';
+        $dsn = 'pgsql:host=foo;dbname=\'bar\';client_encoding=\'utf8\';application_name=\'Laravel App\'';
         $config = ['host' => 'foo', 'database' => 'bar', 'charset' => 'utf8', 'application_name' => 'Laravel App'];
         $connector = $this->getMockBuilder(PostgresConnector::class)->onlyMethods(['createConnection', 'getOptions'])->getMock();
         $connection = m::mock(stdClass::class);
         $connector->expects($this->once())->method('getOptions')->with($this->equalTo($config))->willReturn(['options']);
         $connector->expects($this->once())->method('createConnection')->with($this->equalTo($dsn), $this->equalTo($config), $this->equalTo(['options']))->willReturn($connection);
         $statement = m::mock(PDOStatement::class);
-        $connection->shouldReceive('prepare')->once()->with('set names \'utf8\'')->andReturn($statement);
-        $connection->shouldReceive('prepare')->once()->with('set application_name to \'Laravel App\'')->andReturn($statement);
-        $statement->shouldReceive('execute')->twice();
+        $connection->shouldReceive('prepare')->zeroOrMoreTimes()->andReturn($statement);
+        $statement->shouldReceive('execute')->zeroOrMoreTimes();
+        $result = $connector->connect($config);
+
+        $this->assertSame($result, $connection);
+    }
+
+    public function testPostgresApplicationUseAlternativeDatabaseName()
+    {
+        $dsn = 'pgsql:dbname=\'baz\'';
+        $config = ['database' => 'bar', 'connect_via_database' => 'baz'];
+        $connector = $this->getMockBuilder(PostgresConnector::class)->onlyMethods(['createConnection', 'getOptions'])->getMock();
+        $connection = m::mock(stdClass::class);
+        $connector->expects($this->once())->method('getOptions')->with($this->equalTo($config))->willReturn(['options']);
+        $connector->expects($this->once())->method('createConnection')->with($this->equalTo($dsn), $this->equalTo($config), $this->equalTo(['options']))->willReturn($connection);
+        $statement = m::mock(PDOStatement::class);
+        $connection->shouldReceive('prepare')->zeroOrMoreTimes()->andReturn($statement);
+        $statement->shouldReceive('execute')->zeroOrMoreTimes();
+        $result = $connector->connect($config);
+
+        $this->assertSame($result, $connection);
+    }
+
+    public function testPostgresApplicationUseAlternativeDatabaseNameAndPort()
+    {
+        $dsn = 'pgsql:dbname=\'baz\';port=2345';
+        $config = ['database' => 'bar', 'connect_via_database' => 'baz', 'port' => 5432, 'connect_via_port' => 2345];
+        $connector = $this->getMockBuilder(PostgresConnector::class)->onlyMethods(['createConnection', 'getOptions'])->getMock();
+        $connection = m::mock(stdClass::class);
+        $connector->expects($this->once())->method('getOptions')->with($this->equalTo($config))->willReturn(['options']);
+        $connector->expects($this->once())->method('createConnection')->with($this->equalTo($dsn), $this->equalTo($config), $this->equalTo(['options']))->willReturn($connection);
+        $statement = m::mock(PDOStatement::class);
+        $connection->shouldReceive('prepare')->zeroOrMoreTimes()->andReturn($statement);
+        $statement->shouldReceive('execute')->zeroOrMoreTimes();
         $result = $connector->connect($config);
 
         $this->assertSame($result, $connection);
