@@ -22,22 +22,18 @@ class DatabaseEloquentMorphToTest extends TestCase
 
     public function testLookupDictionaryIsProperlyConstructedForEnums()
     {
-        if (version_compare(PHP_VERSION, '8.1') < 0) {
-            $this->markTestSkipped('PHP 8.1 is required');
+        $relation = $this->getRelation();
+        $relation->addEagerConstraints([
+            $one = (object) ['morph_type' => 'morph_type_2', 'foreign_key' => TestEnum::test],
+        ]);
+        $dictionary = $relation->getDictionary();
+        $relation->getDictionary();
+        $enumKey = TestEnum::test;
+        if (isset($enumKey->value)) {
+            $value = $dictionary['morph_type_2'][$enumKey->value][0]->foreign_key;
+            $this->assertEquals(TestEnum::test, $value);
         } else {
-            $relation = $this->getRelation();
-            $relation->addEagerConstraints([
-                $one = (object) ['morph_type' => 'morph_type_2', 'foreign_key' => TestEnum::test],
-            ]);
-            $dictionary = $relation->getDictionary();
-            $relation->getDictionary();
-            $enumKey = TestEnum::test;
-            if (isset($enumKey->value)) {
-                $value = $dictionary['morph_type_2'][$enumKey->value][0]->foreign_key;
-                $this->assertEquals(TestEnum::test, $value);
-            } else {
-                $this->fail('An enum should contain value property');
-            }
+            $this->fail('An enum should contain value property');
         }
     }
 
