@@ -4,8 +4,6 @@ namespace Illuminate\Tests\Filesystem;
 
 use Illuminate\Contracts\Filesystem\FileNotFoundException;
 use Illuminate\Filesystem\Filesystem;
-use Illuminate\Filesystem\FilesystemManager;
-use Illuminate\Foundation\Application;
 use Illuminate\Support\LazyCollection;
 use Illuminate\Testing\Assert;
 use Mockery as m;
@@ -348,6 +346,20 @@ class FilesystemTest extends TestCase
         $files->getRequire(self::$tempDir.'/file.php');
     }
 
+    public function testJsonReturnsDecodedJsonData()
+    {
+        file_put_contents(self::$tempDir.'/file.json', '{"foo": "bar"}');
+        $files = new Filesystem;
+        $this->assertSame(['foo' => 'bar'], $files->json(self::$tempDir.'/file.json'));
+    }
+
+    public function testJsonReturnsNullIfJsonDataIsInvalid()
+    {
+        file_put_contents(self::$tempDir.'/file.json', '{"foo":');
+        $files = new Filesystem;
+        $this->assertNull($files->json(self::$tempDir.'/file.json'));
+    }
+
     public function testAppendAddsDataToFile()
     {
         file_put_contents(self::$tempDir.'/file.txt', 'foo');
@@ -452,6 +464,29 @@ class FilesystemTest extends TestCase
         $this->assertFalse($files->isReadable(self::$tempDir.'/doesnotexist.txt'));
     }
 
+    public function testIsDirEmpty()
+    {
+        mkdir(self::$tempDir.'/foo-dir');
+        file_put_contents(self::$tempDir.'/foo-dir/.hidden', 'foo');
+        mkdir(self::$tempDir.'/bar-dir');
+        file_put_contents(self::$tempDir.'/bar-dir/foo.txt', 'foo');
+        mkdir(self::$tempDir.'/baz-dir');
+        mkdir(self::$tempDir.'/baz-dir/.hidden');
+        mkdir(self::$tempDir.'/quz-dir');
+        mkdir(self::$tempDir.'/quz-dir/not-hidden');
+
+        $files = new Filesystem;
+
+        $this->assertTrue($files->isEmptyDirectory(self::$tempDir.'/foo-dir', true));
+        $this->assertFalse($files->isEmptyDirectory(self::$tempDir.'/foo-dir'));
+        $this->assertFalse($files->isEmptyDirectory(self::$tempDir.'/bar-dir', true));
+        $this->assertFalse($files->isEmptyDirectory(self::$tempDir.'/bar-dir'));
+        $this->assertTrue($files->isEmptyDirectory(self::$tempDir.'/baz-dir', true));
+        $this->assertFalse($files->isEmptyDirectory(self::$tempDir.'/baz-dir'));
+        $this->assertFalse($files->isEmptyDirectory(self::$tempDir.'/quz-dir', true));
+        $this->assertFalse($files->isEmptyDirectory(self::$tempDir.'/quz-dir'));
+    }
+
     public function testGlobFindsFiles()
     {
         file_put_contents(self::$tempDir.'/foo.txt', 'foo');
@@ -546,6 +581,21 @@ class FilesystemTest extends TestCase
         $this->assertEquals($data, file_get_contents(self::$tempDir.'/text/foo2.txt'));
     }
 
+    public function testHasSameHashChecksFileHashes()
+    {
+        $filesystem = new Filesystem;
+
+        mkdir(self::$tempDir.'/text');
+        file_put_contents(self::$tempDir.'/text/foo.txt', 'contents');
+        file_put_contents(self::$tempDir.'/text/foo2.txt', 'contents');
+        file_put_contents(self::$tempDir.'/text/foo3.txt', 'invalid');
+
+        $this->assertTrue($filesystem->hasSameHash(self::$tempDir.'/text/foo.txt', self::$tempDir.'/text/foo2.txt'));
+        $this->assertFalse($filesystem->hasSameHash(self::$tempDir.'/text/foo.txt', self::$tempDir.'/text/foo3.txt'));
+        $this->assertFalse($filesystem->hasSameHash(self::$tempDir.'/text/foo4.txt', self::$tempDir.'/text/foo.txt'));
+        $this->assertFalse($filesystem->hasSameHash(self::$tempDir.'/text/foo.txt', self::$tempDir.'/text/foo4.txt'));
+    }
+
     public function testIsFileChecksFilesProperly()
     {
         $filesystem = new Filesystem;
@@ -574,32 +624,19 @@ class FilesystemTest extends TestCase
         $this->assertContainsOnlyInstancesOf(SplFileInfo::class, $files->allFiles(self::$tempDir));
     }
 
-    /**
-     * @requires extension ftp
-     */
-    public function testCreateFtpDriver()
+    public function testHashWithDefaultValue()
     {
-        $filesystem = new FilesystemManager(new Application);
-
-        $driver = $filesystem->createFtpDriver([
-            'host' => 'ftp.example.com',
-            'username' => 'admin',
-            'permPublic' => 0700,
-            'unsupportedParam' => true,
-        ]);
-
-        /** @var \League\Flysystem\Adapter\Ftp $adapter */
-        $adapter = $driver->getAdapter();
-        $this->assertEquals(0700, $adapter->getPermPublic());
-        $this->assertSame('ftp.example.com', $adapter->getHost());
-        $this->assertSame('admin', $adapter->getUsername());
+        file_put_contents(self::$tempDir.'/foo.txt', 'foo');
+        $filesystem = new Filesystem;
+        $this->assertSame('acbd18db4cc2f85cedef654fccc4a4d8', $filesystem->hash(self::$tempDir.'/foo.txt'));
     }
 
     public function testHash()
     {
         file_put_contents(self::$tempDir.'/foo.txt', 'foo');
         $filesystem = new Filesystem;
-        $this->assertSame('acbd18db4cc2f85cedef654fccc4a4d8', $filesystem->hash(self::$tempDir.'/foo.txt'));
+        $this->assertSame('0beec7b5ea3f0fdbc95d0dd47f3c5bc275da8a33', $filesystem->hash(self::$tempDir.'/foo.txt', 'sha1'));
+        $this->assertSame('76d3bc41c9f588f7fcd0d5bf4718f8f84b1c41b20882703100b9eb9413807c01', $filesystem->hash(self::$tempDir.'/foo.txt', 'sha3-256'));
     }
 
     /**
