@@ -67,6 +67,7 @@ class FoundationApplicationTest extends TestCase
         $app->register($provider = new class($app) extends ServiceProvider
         {
             public $singletons = [
+                NonContractBackedClass::class,
                 AbstractClass::class => ConcreteClass::class,
             ];
         });
@@ -77,6 +78,11 @@ class FoundationApplicationTest extends TestCase
 
         $this->assertInstanceOf(ConcreteClass::class, $instance);
         $this->assertSame($instance, $app->make(AbstractClass::class));
+
+        $instance = $app->make(NonContractBackedClass::class);
+
+        $this->assertInstanceOf(NonContractBackedClass::class, $instance);
+        $this->assertSame($instance, $app->make(NonContractBackedClass::class));
     }
 
     public function testServiceProvidersAreCorrectlyRegisteredWhenRegisterMethodIsNotFilled()
@@ -493,6 +499,31 @@ class FoundationApplicationTest extends TestCase
             $_SERVER['APP_EVENTS_CACHE']
         );
     }
+
+    public function testMacroable(): void
+    {
+        $app = new Application;
+        $app['env'] = 'foo';
+
+        $app->macro('foo', function () {
+            return $this->environment('foo');
+        });
+
+        $this->assertTrue($app->foo());
+
+        $app['env'] = 'bar';
+
+        $this->assertFalse($app->foo());
+    }
+
+    public function testUseConfigPath(): void
+    {
+        $app = new Application;
+        $app->useConfigPath(__DIR__.'/fixtures/config');
+        $app->bootstrapWith([\Illuminate\Foundation\Bootstrap\LoadConfiguration::class]);
+
+        $this->assertSame('bar', $app->make('config')->get('app.foo'));
+    }
 }
 
 class ApplicationBasicServiceProviderStub extends ServiceProvider
@@ -613,6 +644,11 @@ class ConcreteClass extends AbstractClass
     //
 }
 
+class NonContractBackedClass
+{
+    //
+}
+
 class ConcreteTerminator
 {
     public static $counter = 0;
