@@ -120,7 +120,7 @@ class FoundationFormRequestTest extends TestCase
         $this->createRequest([], FoundationTestFormRequestHooks::class)->validateResolved();
     }
 
-    public function test_after_validation_runs_after_validation()
+    public function testAfterValidationRunsAfterValidation()
     {
         $request = $this->createRequest([], FoundationTestFormRequestHooks::class);
 
@@ -129,6 +129,106 @@ class FoundationFormRequestTest extends TestCase
         $this->assertEquals(['name' => 'Adam'], $request->all());
     }
 
+    public function testValidatedMethodReturnsOnlyRequestedValidatedData()
+    {
+        $request = $this->createRequest(['name' => 'specified', 'with' => 'extras']);
+
+        $request->validateResolved();
+
+        $this->assertSame('specified', $request->validated('name'));
+    }
+
+    public function testValidatedMethodReturnsOnlyRequestedNestedValidatedData()
+    {
+        $payload = ['nested' => ['foo' => 'bar', 'baz' => ''], 'array' => [1, 2]];
+
+        $request = $this->createRequest($payload, FoundationTestFormRequestNestedStub::class);
+
+        $request->validateResolved();
+
+        $this->assertSame('bar', $request->validated('nested.foo'));
+    }
+
+    public function testAfterMethod()
+    {
+        $request = new class extends FormRequest
+        {
+            public $value = 'value-from-request';
+
+            public function rules()
+            {
+                return [];
+            }
+
+            protected function failedValidation(Validator $validator)
+            {
+                throw new class($validator) extends Exception
+                {
+                    public function __construct(public $validator)
+                    {
+                        //
+                    }
+                };
+            }
+
+            public function after(InjectedDependency $dep)
+            {
+                return [
+                    new AfterValidationRule($dep->value),
+                    new InvokableAfterValidationRule($this->value),
+                    fn ($validator) => $validator->errors()->add('closure', 'true'),
+                ];
+            }
+        };
+        $request->setContainer($container = new Container);
+        $container->instance(\Illuminate\Contracts\Validation\Factory::class, (new \Illuminate\Validation\Factory(
+            new \Illuminate\Translation\Translator(new \Illuminate\Translation\ArrayLoader(), 'en')
+        ))->setContainer($container));
+        $container->instance(InjectedDependency::class, new InjectedDependency('value-from-dependency'));
+
+        $messages = [];
+
+        try {
+            $request->validateResolved();
+            $this->fail();
+        } catch (Exception $e) {
+            if (property_exists($e, 'validator')) {
+                $messages = $e->validator->messages()->messages();
+            }
+        }
+
+        $this->assertSame([
+            'after' => ['value-from-dependency'],
+            'invokable' => ['value-from-request'],
+            'closure' => ['true'],
+        ], $messages);
+    }
+
+    public function testRequestCanPassWithoutRulesMethod()
+    {
+        $request = $this->createRequest([], FoundationTestFormRequestWithoutRulesMethod::class);
+
+        $request->validateResolved();
+
+        $this->assertEquals([], $request->all());
+    }
+
+    public function testRequestWithGetRules()
+    {
+        FoundationTestFormRequestWithGetRules::$useRuleSet = 'a';
+        $request = $this->createRequest(['a' => 1], FoundationTestFormRequestWithGetRules::class);
+
+        $request->validateResolved();
+        $this->assertEquals(['a' => 1], $request->all());
+
+        $this->expectException(ValidationException::class);
+        FoundationTestFormRequestWithGetRules::$useRuleSet = 'b';
+
+        $request = $this->createRequest(['a' => 1], FoundationTestFormRequestWithGetRules::class);
+
+        $request->validateResolved();
+    }
+
     /**
      * Catch the given exception thrown from the executor, and return it.
      *
@@ -357,3 +457,62 @@ class FoundationTestFormRequestPassesWithResponseStub extends FormRequest
         return Response::allow('baz');
     }
 }
+
+class InvokableAfterValidationRule
+{
+    public function __construct(private $value)
+    {
+    }
+
+    public function __invoke($validator)
+    {
+        $validator->errors()->add('invokable', $this->value);
+    }
+}
+
+class AfterValidationRule
+{
+    public function __construct(private $value)
+    {
+        //
+    }
+
+    public function after($validator)
+    {
+        $validator->errors()->add('after', $this->value);
+    }
+}
+
+class InjectedDependency
+{
+    public function __construct(public $value)
+    {
+        //
+    }
+}
+
+class FoundationTestFormRequestWithoutRulesMethod extends FormRequest
+{
+    public function authorize()
+    {
+        return true;
+    }
+}
+
+class FoundationTestFormRequestWithGetRules extends FormRequest
+{
+    public static $useRuleSet = 'a';
+
+    protected function validationRules(): array
+    {
+        if (self::$useRuleSet === 'a') {
+            return [
+                'a' => ['required', 'int', 'min:1'],
+            ];
+        } else {
+            return [
+                'a' => ['required', 'int', 'min:2'],
+            ];
+        }
+    }
+}
