@@ -2,14 +2,49 @@
 
 namespace Illuminate\Tests\Hashing;
 
+use Illuminate\Config\Repository as Config;
+use Illuminate\Container\Container;
 use Illuminate\Hashing\Argon2IdHasher;
 use Illuminate\Hashing\ArgonHasher;
 use Illuminate\Hashing\BcryptHasher;
+use Illuminate\Hashing\HashManager;
 use PHPUnit\Framework\TestCase;
 use RuntimeException;
 
 class HasherTest extends TestCase
 {
+    public $hashManager;
+
+    protected function setUp(): void
+    {
+        parent::setUp();
+
+        $container = Container::setInstance(new Container);
+        $container->singleton('config', fn () => new Config());
+
+        $this->hashManager = new HashManager($container);
+    }
+
+    public function testEmptyHashedValueReturnsFalse()
+    {
+        $hasher = new BcryptHasher();
+        $this->assertFalse($hasher->check('password', ''));
+        $hasher = new ArgonHasher();
+        $this->assertFalse($hasher->check('password', ''));
+        $hasher = new Argon2IdHasher();
+        $this->assertFalse($hasher->check('password', ''));
+    }
+
+    public function testNullHashedValueReturnsFalse()
+    {
+        $hasher = new BcryptHasher();
+        $this->assertFalse($hasher->check('password', null));
+        $hasher = new ArgonHasher();
+        $this->assertFalse($hasher->check('password', null));
+        $hasher = new Argon2IdHasher();
+        $this->assertFalse($hasher->check('password', null));
+    }
+
     public function testBasicBcryptHashing()
     {
         $hasher = new BcryptHasher;
@@ -19,6 +54,8 @@ class HasherTest extends TestCase
         $this->assertFalse($hasher->needsRehash($value));
         $this->assertTrue($hasher->needsRehash($value, ['rounds' => 1]));
         $this->assertSame('bcrypt', password_get_info($value)['algoName']);
+        $this->assertGreaterThanOrEqual(12, password_get_info($value)['options']['cost']);
+        $this->assertTrue($this->hashManager->isHashed($value));
     }
 
     public function testBasicArgon2iHashing()
@@ -30,6 +67,7 @@ class HasherTest extends TestCase
         $this->assertFalse($hasher->needsRehash($value));
         $this->assertTrue($hasher->needsRehash($value, ['threads' => 1]));
         $this->assertSame('argon2i', password_get_info($value)['algoName']);
+        $this->assertTrue($this->hashManager->isHashed($value));
     }
 
     public function testBasicArgon2idHashing()
@@ -41,6 +79,7 @@ class HasherTest extends TestCase
         $this->assertFalse($hasher->needsRehash($value));
         $this->assertTrue($hasher->needsRehash($value, ['threads' => 1]));
         $this->assertSame('argon2id', password_get_info($value)['algoName']);
+        $this->assertTrue($this->hashManager->isHashed($value));
     }
 
     /**
@@ -78,4 +117,9 @@ class HasherTest extends TestCase
         $bcryptHashed = $bcryptHasher->make('password');
         (new Argon2IdHasher(['verify' => true]))->check('password', $bcryptHashed);
     }
+
+    public function testIsHashedWithNonHashedValue()
+    {
+        $this->assertFalse($this->hashManager->isHashed('foo'));
+    }
 }
