@@ -3,23 +3,26 @@
 namespace Illuminate\Tests\Integration\Auth\Fixtures;
 
 use Illuminate\Foundation\Auth\User as Authenticatable;
+use Illuminate\Notifications\Notifiable;
 
 class AuthenticationTestUser extends Authenticatable
 {
+    use Notifiable;
+
     public $table = 'users';
     public $timestamps = false;
 
     /**
      * The attributes that aren't mass assignable.
      *
-     * @var array
+     * @var string[]
      */
-    protected $guarded = ['id'];
+    protected $guarded = [];
 
     /**
      * The attributes that should be hidden for arrays.
      *
-     * @var array
+     * @var string[]
      */
     protected $hidden = [
         'password', 'remember_token',
