@@ -4,7 +4,6 @@ namespace Illuminate\Tests\Integration\Cache;
 
 use Exception;
 use Illuminate\Foundation\Testing\Concerns\InteractsWithRedis;
-use Illuminate\Support\Carbon;
 use Illuminate\Support\Facades\Cache;
 use Orchestra\Testbench\TestCase;
 
@@ -50,8 +49,6 @@ class RedisCacheLockTest extends TestCase
 
     public function testRedisLocksCanBlockForSeconds()
     {
-        Carbon::setTestNow();
-
         Cache::store('redis')->lock('foo')->forceRelease();
         $this->assertSame('taylor', Cache::store('redis')->lock('foo', 10)->block(1, function () {
             return 'taylor';
@@ -87,7 +84,7 @@ class RedisCacheLockTest extends TestCase
             $firstLock->block(1, function () {
                 throw new Exception('failed');
             });
-        } catch (Exception $e) {
+        } catch (Exception) {
             // Not testing the exception, just testing the lock
             // is released regardless of the how the exception
             // thrown by the callback was handled.
@@ -111,4 +108,27 @@ class RedisCacheLockTest extends TestCase
 
         $this->assertTrue(Cache::store('redis')->lock('foo')->get());
     }
+
+    public function testOwnerStatusCanBeCheckedAfterRestoringLock()
+    {
+        Cache::store('redis')->lock('foo')->forceRelease();
+
+        $firstLock = Cache::store('redis')->lock('foo', 10);
+        $this->assertTrue($firstLock->get());
+        $owner = $firstLock->owner();
+
+        $secondLock = Cache::store('redis')->restoreLock('foo', $owner);
+        $this->assertTrue($secondLock->isOwnedByCurrentProcess());
+    }
+
+    public function testOtherOwnerDoesNotOwnLockAfterRestore()
+    {
+        Cache::store('redis')->lock('foo')->forceRelease();
+
+        $firstLock = Cache::store('redis')->lock('foo', 10);
+        $this->assertTrue($firstLock->get());
+
+        $secondLock = Cache::store('redis')->restoreLock('foo', 'other_owner');
+        $this->assertFalse($secondLock->isOwnedByCurrentProcess());
+    }
 }
