@@ -15,7 +15,14 @@ use Illuminate\Tests\Integration\Database\DatabaseTestCase;
 
 class EloquentBelongsToManyTest extends DatabaseTestCase
 {
-    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
+    protected function tearDown(): void
+    {
+        parent::tearDown();
+
+        Carbon::setTestNow(null);
+    }
+
+    protected function afterRefreshingDatabase()
     {
         Schema::create('users', function (Blueprint $table) {
             $table->increments('id');
@@ -34,6 +41,14 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         Schema::create('tags', function (Blueprint $table) {
             $table->increments('id');
             $table->string('name');
+            $table->string('type')->nullable();
+            $table->timestamps();
+        });
+
+        Schema::create('unique_tags', function (Blueprint $table) {
+            $table->increments('id');
+            $table->string('name')->unique();
+            $table->string('type')->nullable();
             $table->timestamps();
         });
 
@@ -52,6 +67,14 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
             $table->string('flag')->default('')->nullable();
             $table->timestamps();
         });
+
+        Schema::create('posts_unique_tags', function (Blueprint $table) {
+            $table->integer('post_id');
+            $table->integer('tag_id')->default(0);
+            $table->string('tag_name')->default('')->nullable();
+            $table->string('flag')->default('')->nullable();
+            $table->timestamps();
+        });
     }
 
     public function testBasicCreateAndRetrieve()
@@ -130,7 +153,7 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $post->tagsWithCustomPivot()->attach($tag->id);
 
         $this->assertInstanceOf(PostTagPivot::class, $post->tagsWithCustomPivot[0]->pivot);
-        $this->assertEquals('1507630210', $post->tagsWithCustomPivot[0]->pivot->created_at);
+        $this->assertSame('1507630210', $post->tagsWithCustomPivot[0]->pivot->created_at);
 
         $this->assertInstanceOf(PostTagPivot::class, $post->tagsWithCustomPivotClass[0]->pivot);
         $this->assertSame('posts_tags', $post->tagsWithCustomPivotClass()->getTable());
@@ -201,7 +224,6 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         );
     }
 
-    /** @group SkipMSSQL */
     public function testCustomPivotClassUpdatesTimestamps()
     {
         Carbon::setTestNow('2017-10-10 10:10:10');
@@ -225,8 +247,14 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         );
         foreach ($post->tagsWithCustomExtraPivot as $tag) {
             $this->assertSame('exclude', $tag->pivot->flag);
-            $this->assertEquals('2017-10-10 10:10:10', $tag->pivot->getAttributes()['created_at']);
-            $this->assertEquals('2017-10-10 10:10:20', $tag->pivot->getAttributes()['updated_at']); // +10 seconds
+
+            if ($this->driver === 'sqlsrv') {
+                $this->assertSame('2017-10-10 10:10:10.000', $tag->pivot->getAttributes()['created_at']);
+                $this->assertSame('2017-10-10 10:10:20.000', $tag->pivot->getAttributes()['updated_at']); // +10 seconds
+            } else {
+                $this->assertSame('2017-10-10 10:10:10', $tag->pivot->getAttributes()['created_at']);
+                $this->assertSame('2017-10-10 10:10:20', $tag->pivot->getAttributes()['updated_at']); // +10 seconds
+            }
         }
     }
 
@@ -349,6 +377,48 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertCount(2, $post->tags()->findMany(new Collection([$tag->id, $tag2->id])));
     }
 
+    public function testFindMethodStringyKey()
+    {
+        Schema::create('post_string_key', function (Blueprint $table) {
+            $table->string('id', 1)->primary();
+            $table->string('title', 10);
+        });
+
+        Schema::create('tag_string_key', function (Blueprint $table) {
+            $table->string('id', 1)->primary();
+            $table->string('title', 10);
+        });
+
+        Schema::create('post_tag_string_key', function (Blueprint $table) {
+            $table->id();
+            $table->string('post_id', 1);
+            $table->string('tag_id', 1);
+        });
+
+        $post = PostStringPrimaryKey::query()->create([
+            'id' => 'a',
+            'title' => Str::random(10),
+        ]);
+
+        $tag = TagStringPrimaryKey::query()->create([
+            'id' => 'b',
+            'title' => Str::random(10),
+        ]);
+
+        $tag2 = TagStringPrimaryKey::query()->create([
+            'id' => 'c',
+            'title' => Str::random(10),
+        ]);
+
+        $post->tags()->attach(TagStringPrimaryKey::all());
+
+        $this->assertEquals($tag2->name, $post->tags()->find($tag2->id)->name);
+        $this->assertCount(0, $post->tags()->findMany([]));
+        $this->assertCount(2, $post->tags()->findMany([$tag->id, $tag2->id]));
+        $this->assertCount(0, $post->tags()->findMany(new Collection));
+        $this->assertCount(2, $post->tags()->findMany(new Collection([$tag->id, $tag2->id])));
+    }
+
     public function testFindOrFailMethod()
     {
         $this->expectException(ModelNotFoundException::class);
@@ -405,6 +475,77 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertInstanceOf(Tag::class, $post->tags()->findOrNew(666));
     }
 
+    public function testFindOrMethod()
+    {
+        $post = Post::create(['title' => Str::random()]);
+        $post->tags()->create(['name' => Str::random()]);
+
+        $result = $post->tags()->findOr(1, fn () => 'callback result');
+        $this->assertInstanceOf(Tag::class, $result);
+        $this->assertSame(1, $result->id);
+        $this->assertNotNull($result->name);
+
+        $result = $post->tags()->findOr(1, ['id'], fn () => 'callback result');
+        $this->assertInstanceOf(Tag::class, $result);
+        $this->assertSame(1, $result->id);
+        $this->assertNull($result->name);
+
+        $result = $post->tags()->findOr(2, fn () => 'callback result');
+        $this->assertSame('callback result', $result);
+    }
+
+    public function testFindOrMethodWithMany()
+    {
+        $post = Post::create(['title' => Str::random()]);
+        $post->tags()->createMany([
+            ['name' => Str::random()],
+            ['name' => Str::random()],
+        ]);
+
+        $result = $post->tags()->findOr([1, 2], fn () => 'callback result');
+        $this->assertInstanceOf(Collection::class, $result);
+        $this->assertSame(1, $result[0]->id);
+        $this->assertSame(2, $result[1]->id);
+        $this->assertNotNull($result[0]->name);
+        $this->assertNotNull($result[1]->name);
+
+        $result = $post->tags()->findOr([1, 2], ['id'], fn () => 'callback result');
+        $this->assertInstanceOf(Collection::class, $result);
+        $this->assertSame(1, $result[0]->id);
+        $this->assertSame(2, $result[1]->id);
+        $this->assertNull($result[0]->name);
+        $this->assertNull($result[1]->name);
+
+        $result = $post->tags()->findOr([1, 2, 3], fn () => 'callback result');
+        $this->assertSame('callback result', $result);
+    }
+
+    public function testFindOrMethodWithManyUsingCollection()
+    {
+        $post = Post::create(['title' => Str::random()]);
+        $post->tags()->createMany([
+            ['name' => Str::random()],
+            ['name' => Str::random()],
+        ]);
+
+        $result = $post->tags()->findOr(new Collection([1, 2]), fn () => 'callback result');
+        $this->assertInstanceOf(Collection::class, $result);
+        $this->assertSame(1, $result[0]->id);
+        $this->assertSame(2, $result[1]->id);
+        $this->assertNotNull($result[0]->name);
+        $this->assertNotNull($result[1]->name);
+
+        $result = $post->tags()->findOr(new Collection([1, 2]), ['id'], fn () => 'callback result');
+        $this->assertInstanceOf(Collection::class, $result);
+        $this->assertSame(1, $result[0]->id);
+        $this->assertSame(2, $result[1]->id);
+        $this->assertNull($result[0]->name);
+        $this->assertNull($result[1]->name);
+
+        $result = $post->tags()->findOr(new Collection([1, 2, 3]), fn () => 'callback result');
+        $this->assertSame('callback result', $result);
+    }
+
     public function testFirstOrNewMethod()
     {
         $post = Post::create(['title' => Str::random()]);
@@ -419,6 +560,19 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertInstanceOf(Tag::class, $post->tags()->firstOrNew(['id' => 666]));
     }
 
+    // public function testFirstOrNewUnrelatedExisting()
+    // {
+    //     $post = Post::create(['title' => Str::random()]);
+
+    //     $name = Str::random();
+    //     $tag = Tag::create(['name' => $name]);
+
+    //     $postTag = $post->tags()->firstOrNew(['name' => $name]);
+    //     $this->assertTrue($postTag->exists);
+    //     $this->assertTrue($postTag->is($tag));
+    //     $this->assertTrue($tag->is($post->tags()->first()));
+    // }
+
     public function testFirstOrCreateMethod()
     {
         $post = Post::create(['title' => Str::random()]);
@@ -434,6 +588,122 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertNotNull($new->id);
     }
 
+    public function testFirstOrCreateUnrelatedExisting()
+    {
+        $post = Post::create(['title' => Str::random()]);
+
+        $name = Str::random();
+        $tag = Tag::create(['name' => $name]);
+
+        $postTag = $post->tags()->firstOrCreate(['name' => $name]);
+        $this->assertTrue($postTag->exists);
+        $this->assertTrue($postTag->is($tag));
+        $this->assertTrue($tag->is($post->tags()->first()));
+    }
+
+    public function testCreateOrFirst()
+    {
+        $post = Post::create(['title' => Str::random()]);
+
+        $tag = UniqueTag::create(['name' => Str::random()]);
+
+        $post->tagsUnique()->attach(UniqueTag::all());
+
+        $this->assertEquals($tag->id, $post->tagsUnique()->createOrFirst(['name' => $tag->name])->id);
+
+        $new = $post->tagsUnique()->createOrFirst(['name' => 'wavez']);
+        $this->assertSame('wavez', $new->name);
+        $this->assertNotNull($new->id);
+    }
+
+    public function testCreateOrFirstUnrelatedExisting()
+    {
+        $post = Post::create(['title' => Str::random()]);
+
+        $name = Str::random();
+        $tag = UniqueTag::create(['name' => $name]);
+
+        $postTag = $post->tagsUnique()->createOrFirst(['name' => $name]);
+        $this->assertTrue($postTag->exists);
+        $this->assertTrue($postTag->is($tag));
+        $this->assertTrue($tag->is($post->tagsUnique()->first()));
+    }
+
+    public function testCreateOrFirstWithinTransaction()
+    {
+        $post = Post::create(['title' => Str::random()]);
+
+        $tag = UniqueTag::create(['name' => Str::random()]);
+
+        $post->tagsUnique()->attach(UniqueTag::all());
+
+        DB::transaction(function () use ($tag, $post) {
+            $this->assertEquals($tag->id, $post->tagsUnique()->createOrFirst(['name' => $tag->name])->id);
+        });
+    }
+
+    public function testFirstOrNewMethodWithValues()
+    {
+        $post = Post::create(['title' => Str::random()]);
+        $tag = Tag::create(['name' => Str::random()]);
+        $post->tags()->attach(Tag::all());
+
+        $existing = $post->tags()->firstOrNew(
+            ['name' => $tag->name],
+            ['type' => 'featured']
+        );
+
+        $this->assertEquals($tag->id, $existing->id);
+        $this->assertNotEquals('foo', $existing->name);
+
+        $new = $post->tags()->firstOrNew(
+            ['name' => 'foo'],
+            ['type' => 'featured']
+        );
+
+        $this->assertSame('foo', $new->name);
+        $this->assertSame('featured', $new->type);
+
+        $new = $post->tags()->firstOrNew(
+            ['name' => 'foo'],
+            ['name' => 'bar']
+        );
+
+        $this->assertSame('bar', $new->name);
+    }
+
+    public function testFirstOrCreateMethodWithValues()
+    {
+        $post = Post::create(['title' => Str::random()]);
+        $tag = Tag::create(['name' => Str::random()]);
+        $post->tags()->attach(Tag::all());
+
+        $existing = $post->tags()->firstOrCreate(
+            ['name' => $tag->name],
+            ['type' => 'featured']
+        );
+
+        $this->assertEquals($tag->id, $existing->id);
+        $this->assertNotEquals('foo', $existing->name);
+
+        $new = $post->tags()->firstOrCreate(
+            ['name' => 'foo'],
+            ['type' => 'featured']
+        );
+
+        $this->assertSame('foo', $new->name);
+        $this->assertSame('featured', $new->type);
+        $this->assertNotNull($new->id);
+
+        $new = $post->tags()->firstOrCreate(
+            ['name' => 'qux'],
+            ['name' => 'bar']
+        );
+
+        $this->assertSame('bar', $new->name);
+        $this->assertNotNull($new->id);
+    }
+
     public function testUpdateOrCreateMethod()
     {
         $post = Post::create(['title' => Str::random()]);
@@ -449,6 +719,32 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertNotNull($post->tags()->whereName('dives')->first());
     }
 
+    public function testUpdateOrCreateUnrelatedExisting()
+    {
+        $post = Post::create(['title' => Str::random()]);
+
+        $tag = Tag::create(['name' => 'foo']);
+
+        $postTag = $post->tags()->updateOrCreate(['name' => 'foo'], ['name' => 'wavez']);
+        $this->assertTrue($postTag->exists);
+        $this->assertTrue($postTag->is($tag));
+        $this->assertSame('wavez', $tag->fresh()->name);
+        $this->assertSame('wavez', $postTag->name);
+        $this->assertTrue($tag->is($post->tags()->first()));
+    }
+
+    public function testUpdateOrCreateMethodCreate()
+    {
+        $post = Post::create(['title' => Str::random()]);
+
+        $post->tags()->updateOrCreate(['name' => 'wavez'], ['type' => 'featured']);
+
+        $tag = $post->tags()->whereType('featured')->first();
+
+        $this->assertNotNull($tag);
+        $this->assertSame('wavez', $tag->name);
+    }
+
     public function testSyncMethod()
     {
         $post = Post::create(['title' => Str::random()]);
@@ -597,39 +893,37 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertNotSame('2017-10-10 10:10:10', $tag->fresh()->updated_at->toDateTimeString());
     }
 
-    /** @group SkipMSSQL */
     public function testCanRetrieveRelatedIds()
     {
         $post = Post::create(['title' => Str::random()]);
 
         DB::table('tags')->insert([
-            ['id' => 200, 'name' => 'excluded'],
-            ['id' => 300, 'name' => Str::random()],
+            ['name' => 'excluded'],
+            ['name' => Str::random()],
         ]);
 
         DB::table('posts_tags')->insert([
-            ['post_id' => $post->id, 'tag_id' => 200, 'flag' => ''],
-            ['post_id' => $post->id, 'tag_id' => 300, 'flag' => 'exclude'],
-            ['post_id' => $post->id, 'tag_id' => 400, 'flag' => ''],
+            ['post_id' => $post->id, 'tag_id' => 1, 'flag' => ''],
+            ['post_id' => $post->id, 'tag_id' => 2, 'flag' => 'exclude'],
+            ['post_id' => $post->id, 'tag_id' => 3, 'flag' => ''],
         ]);
 
-        $this->assertEquals([200, 400], $post->tags()->allRelatedIds()->toArray());
+        $this->assertEquals([1, 3], $post->tags()->allRelatedIds()->toArray());
     }
 
-    /** @group SkipMSSQL */
     public function testCanTouchRelatedModels()
     {
         $post = Post::create(['title' => Str::random()]);
 
         DB::table('tags')->insert([
-            ['id' => 200, 'name' => Str::random()],
-            ['id' => 300, 'name' => Str::random()],
+            ['name' => Str::random()],
+            ['name' => Str::random()],
         ]);
 
         DB::table('posts_tags')->insert([
-            ['post_id' => $post->id, 'tag_id' => 200, 'flag' => ''],
-            ['post_id' => $post->id, 'tag_id' => 300, 'flag' => 'exclude'],
-            ['post_id' => $post->id, 'tag_id' => 400, 'flag' => ''],
+            ['post_id' => $post->id, 'tag_id' => 1, 'flag' => ''],
+            ['post_id' => $post->id, 'tag_id' => 2, 'flag' => 'exclude'],
+            ['post_id' => $post->id, 'tag_id' => 3, 'flag' => ''],
         ]);
 
         Carbon::setTestNow('2017-10-10 10:10:10');
@@ -637,16 +931,19 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $post->tags()->touch();
 
         foreach ($post->tags()->pluck('tags.updated_at') as $date) {
-            $this->assertSame('2017-10-10 10:10:10', $date);
+            if ($this->driver === 'sqlsrv') {
+                $this->assertSame('2017-10-10 10:10:10.000', $date);
+            } else {
+                $this->assertSame('2017-10-10 10:10:10', $date);
+            }
         }
 
-        $this->assertNotSame('2017-10-10 10:10:10', Tag::find(300)->updated_at);
+        $this->assertNotSame('2017-10-10 10:10:10', Tag::find(2)->updated_at);
     }
 
-    /** @group SkipMSSQL */
     public function testWherePivotOnString()
     {
-        $tag = Tag::create(['name' => Str::random()]);
+        $tag = Tag::create(['name' => Str::random()])->fresh();
         $post = Post::create(['title' => Str::random()]);
 
         DB::table('posts_tags')->insert([
@@ -660,10 +957,9 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTag->getAttributes(), $tag->getAttributes());
     }
 
-    /** @group SkipMSSQL */
     public function testFirstWhere()
     {
-        $tag = Tag::create(['name' => 'foo']);
+        $tag = Tag::create(['name' => 'foo'])->fresh();
         $post = Post::create(['title' => Str::random()]);
 
         DB::table('posts_tags')->insert([
@@ -677,10 +973,9 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTag->getAttributes(), $tag->getAttributes());
     }
 
-    /** @group SkipMSSQL */
     public function testWherePivotOnBoolean()
     {
-        $tag = Tag::create(['name' => Str::random()]);
+        $tag = Tag::create(['name' => Str::random()])->fresh();
         $post = Post::create(['title' => Str::random()]);
 
         DB::table('posts_tags')->insert([
@@ -694,10 +989,9 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTag->getAttributes(), $tag->getAttributes());
     }
 
-    /** @group SkipMSSQL */
     public function testWherePivotInMethod()
     {
-        $tag = Tag::create(['name' => Str::random()]);
+        $tag = Tag::create(['name' => Str::random()])->fresh();
         $post = Post::create(['title' => Str::random()]);
 
         DB::table('posts_tags')->insert([
@@ -729,11 +1023,10 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTags->pluck('id')->toArray(), [$tag1->id, $tag3->id]);
     }
 
-    /** @group SkipMSSQL */
     public function testWherePivotNotInMethod()
     {
         $tag1 = Tag::create(['name' => Str::random()]);
-        $tag2 = Tag::create(['name' => Str::random()]);
+        $tag2 = Tag::create(['name' => Str::random()])->fresh();
         $post = Post::create(['title' => Str::random()]);
 
         DB::table('posts_tags')->insert([
@@ -768,11 +1061,10 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTags->pluck('id')->toArray(), [$tag1->id, $tag2->id]);
     }
 
-    /** @group SkipMSSQL */
     public function testWherePivotNullMethod()
     {
         $tag1 = Tag::create(['name' => Str::random()]);
-        $tag2 = Tag::create(['name' => Str::random()]);
+        $tag2 = Tag::create(['name' => Str::random()])->fresh();
         $post = Post::create(['title' => Str::random()]);
 
         DB::table('posts_tags')->insert([
@@ -786,10 +1078,9 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals($relationTag->getAttributes(), $tag2->getAttributes());
     }
 
-    /** @group SkipMSSQL */
     public function testWherePivotNotNullMethod()
     {
-        $tag1 = Tag::create(['name' => Str::random()]);
+        $tag1 = Tag::create(['name' => Str::random()])->fresh();
         $tag2 = Tag::create(['name' => Str::random()]);
         $post = Post::create(['title' => Str::random()]);
 
@@ -911,12 +1202,11 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
         $this->assertEquals(0, $user->postsWithCustomPivot()->first()->pivot->is_draft);
     }
 
-    /** @group SkipMSSQL */
     public function testOrderByPivotMethod()
     {
         $tag1 = Tag::create(['name' => Str::random()]);
-        $tag2 = Tag::create(['name' => Str::random()]);
-        $tag3 = Tag::create(['name' => Str::random()]);
+        $tag2 = Tag::create(['name' => Str::random()])->fresh();
+        $tag3 = Tag::create(['name' => Str::random()])->fresh();
         $tag4 = Tag::create(['name' => Str::random()]);
         $post = Post::create(['title' => Str::random()]);
 
@@ -960,6 +1250,42 @@ class EloquentBelongsToManyTest extends DatabaseTestCase
             })->id
         );
     }
+
+    public function testUpdateOrCreateQueryBuilderIsolation()
+    {
+        $user = User::create(['name' => Str::random()]);
+        $post = Post::create(['title' => Str::random()]);
+
+        $user->postsWithCustomPivot()->attach($post);
+
+        $instance = $user->postsWithCustomPivot()->updateOrCreate(
+            ['uuid' => $post->uuid],
+            ['title' => Str::random()],
+        );
+
+        $this->assertArrayNotHasKey(
+            'user_uuid',
+            $instance->toArray(),
+        );
+    }
+
+    public function testFirstOrCreateQueryBuilderIsolation()
+    {
+        $user = User::create(['name' => Str::random()]);
+        $post = Post::create(['title' => Str::random()]);
+
+        $user->postsWithCustomPivot()->attach($post);
+
+        $instance = $user->postsWithCustomPivot()->firstOrCreate(
+            ['uuid' => $post->uuid],
+            ['title' => Str::random()],
+        );
+
+        $this->assertArrayNotHasKey(
+            'user_uuid',
+            $instance->toArray(),
+        );
+    }
 }
 
 class User extends Model
@@ -993,6 +1319,42 @@ class User extends Model
     }
 }
 
+class PostStringPrimaryKey extends Model
+{
+    public $incrementing = false;
+
+    public $timestamps = false;
+
+    protected $table = 'post_string_key';
+
+    protected $keyType = 'string';
+
+    protected $fillable = ['title', 'id'];
+
+    public function tags()
+    {
+        return $this->belongsToMany(TagStringPrimaryKey::class, 'post_tag_string_key', 'post_id', 'tag_id');
+    }
+}
+
+class TagStringPrimaryKey extends Model
+{
+    public $incrementing = false;
+
+    public $timestamps = false;
+
+    protected $table = 'tag_string_key';
+
+    protected $keyType = 'string';
+
+    protected $fillable = ['title', 'id'];
+
+    public function posts()
+    {
+        return $this->belongsToMany(PostStringPrimaryKey::class, 'post_tag_string_key', 'tag_id', 'post_id');
+    }
+}
+
 class Post extends Model
 {
     public $table = 'posts';
@@ -1024,6 +1386,14 @@ class Post extends Model
             ->wherePivot('flag', '<>', 'exclude');
     }
 
+    public function tagsUnique()
+    {
+        return $this->belongsToMany(UniqueTag::class, 'posts_unique_tags', 'post_id', 'tag_id')
+            ->withPivot('flag')
+            ->withTimestamps()
+            ->wherePivot('flag', '<>', 'exclude');
+    }
+
     public function tagsWithExtraPivot()
     {
         return $this->belongsToMany(Tag::class, 'posts_tags', 'post_id', 'tag_id')
@@ -1079,7 +1449,7 @@ class Tag extends Model
 {
     public $table = 'tags';
     public $timestamps = true;
-    protected $fillable = ['name'];
+    protected $fillable = ['name', 'type'];
 
     public function posts()
     {
@@ -1087,6 +1457,18 @@ class Tag extends Model
     }
 }
 
+class UniqueTag extends Model
+{
+    public $table = 'unique_tags';
+    public $timestamps = true;
+    protected $fillable = ['name', 'type'];
+
+    public function posts()
+    {
+        return $this->belongsToMany(Post::class, 'posts_unique_tags', 'tag_id', 'post_id');
+    }
+}
+
 class TouchingTag extends Model
 {
     public $table = 'tags';
