@@ -12,7 +12,7 @@ use Illuminate\Tests\Integration\Database\DatabaseTestCase;
 
 class EloquentCollectionLoadCountTest extends DatabaseTestCase
 {
-    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
+    protected function afterRefreshingDatabase()
     {
         Schema::create('posts', function (Blueprint $table) {
             $table->increments('id');
@@ -47,9 +47,9 @@ class EloquentCollectionLoadCountTest extends DatabaseTestCase
         $posts->loadCount('comments');
 
         $this->assertCount(1, DB::getQueryLog());
-        $this->assertEquals('2', $posts[0]->comments_count);
-        $this->assertEquals('0', $posts[1]->comments_count);
-        $this->assertEquals('2', $posts[0]->getOriginal('comments_count'));
+        $this->assertSame('2', (string) $posts[0]->comments_count);
+        $this->assertSame('0', (string) $posts[1]->comments_count);
+        $this->assertSame('2', (string) $posts[0]->getOriginal('comments_count'));
     }
 
     public function testLoadCountWithSameModels()
@@ -61,9 +61,9 @@ class EloquentCollectionLoadCountTest extends DatabaseTestCase
         $posts->loadCount('comments');
 
         $this->assertCount(1, DB::getQueryLog());
-        $this->assertEquals('2', $posts[0]->comments_count);
-        $this->assertEquals('0', $posts[1]->comments_count);
-        $this->assertEquals('2', $posts[2]->comments_count);
+        $this->assertSame('2', (string) $posts[0]->comments_count);
+        $this->assertSame('0', (string) $posts[1]->comments_count);
+        $this->assertSame('2', (string) $posts[2]->comments_count);
     }
 
     public function testLoadCountOnDeletedModels()
@@ -75,8 +75,8 @@ class EloquentCollectionLoadCountTest extends DatabaseTestCase
         $posts->loadCount('comments');
 
         $this->assertCount(1, DB::getQueryLog());
-        $this->assertEquals('2', $posts[0]->comments_count);
-        $this->assertEquals('0', $posts[1]->comments_count);
+        $this->assertSame('2', (string) $posts[0]->comments_count);
+        $this->assertSame('0', (string) $posts[1]->comments_count);
     }
 
     public function testLoadCountWithArrayOfRelations()
@@ -88,10 +88,10 @@ class EloquentCollectionLoadCountTest extends DatabaseTestCase
         $posts->loadCount(['comments', 'likes']);
 
         $this->assertCount(1, DB::getQueryLog());
-        $this->assertEquals('2', $posts[0]->comments_count);
-        $this->assertEquals('1', $posts[0]->likes_count);
-        $this->assertEquals('0', $posts[1]->comments_count);
-        $this->assertEquals('0', $posts[1]->likes_count);
+        $this->assertSame('2', (string) $posts[0]->comments_count);
+        $this->assertSame('1', (string) $posts[0]->likes_count);
+        $this->assertSame('0', (string) $posts[1]->comments_count);
+        $this->assertSame('0', (string) $posts[1]->likes_count);
     }
 
     public function testLoadCountDoesNotOverrideAttributesWithDefaultValue()
@@ -102,7 +102,7 @@ class EloquentCollectionLoadCountTest extends DatabaseTestCase
         Collection::make([$post])->loadCount('comments');
 
         $this->assertSame(200, $post->some_default_value);
-        $this->assertEquals('2', $post->comments_count);
+        $this->assertSame('2', (string) $post->comments_count);
     }
 }
 
