@@ -9,7 +9,7 @@ use Illuminate\Support\Facades\Schema;
 
 class EloquentCustomPivotCastTest extends DatabaseTestCase
 {
-    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
+    protected function afterRefreshingDatabase()
     {
         Schema::create('users', function (Blueprint $table) {
             $table->increments('id');
@@ -137,6 +137,17 @@ class EloquentCustomPivotCastTest extends DatabaseTestCase
         $this->assertEquals(['foo1' => 'bar1'], $project->collaborators[0]->pivot->permissions);
         $this->assertEquals(['baz2' => 'bar2'], $project->collaborators[1]->pivot->permissions);
     }
+
+    public function testDefaultAttributesAreRespectedAndCastsAreRespected()
+    {
+        $project = CustomPivotCastTestProject::forceCreate([
+            'name' => 'Test Project',
+        ]);
+
+        $pivot = $project->collaborators()->newPivot();
+
+        $this->assertEquals(['permissions' => ['create', 'update']], $pivot->toArray());
+    }
 }
 
 class CustomPivotCastTestUser extends Model
@@ -160,6 +171,12 @@ class CustomPivotCastTestProject extends Model
 
 class CustomPivotCastTestCollaborator extends Pivot
 {
+    public $timestamps = false;
+
+    protected $attributes = [
+        'permissions' => '["create", "update"]',
+    ];
+
     protected $casts = [
         'permissions' => 'json',
     ];
