@@ -3,6 +3,7 @@
 namespace Illuminate\Tests\Integration\Database\EloquentMorphManyTest;
 
 use Illuminate\Database\Eloquent\Model;
+use Illuminate\Database\Eloquent\Relations\MorphOne;
 use Illuminate\Database\Schema\Blueprint;
 use Illuminate\Support\Carbon;
 use Illuminate\Support\Facades\Schema;
@@ -11,7 +12,7 @@ use Illuminate\Tests\Integration\Database\DatabaseTestCase;
 
 class EloquentMorphManyTest extends DatabaseTestCase
 {
-    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
+    protected function afterRefreshingDatabase()
     {
         Schema::create('posts', function (Blueprint $table) {
             $table->increments('id');
@@ -26,8 +27,6 @@ class EloquentMorphManyTest extends DatabaseTestCase
             $table->string('commentable_type');
             $table->timestamps();
         });
-
-        Carbon::setTestNow(null);
     }
 
     public function testUpdateModelWithDefaultWithCount()
@@ -51,6 +50,25 @@ class EloquentMorphManyTest extends DatabaseTestCase
 
         $this->assertEquals([1], $comments->pluck('id')->all());
     }
+
+    public function testCanMorphOne()
+    {
+        $post = Post::create(['title' => 'Your favorite book by C.S. Lewis']);
+
+        Carbon::setTestNow('1990-02-02 12:00:00');
+        $oldestComment = tap((new Comment(['name' => 'The Allegory Of Love']))->commentable()->associate($post))->save();
+
+        Carbon::setTestNow('2000-07-02 09:00:00');
+        tap((new Comment(['name' => 'The Screwtape Letters']))->commentable()->associate($post))->save();
+
+        Carbon::setTestNow('2022-01-01 00:00:00');
+        $latestComment = tap((new Comment(['name' => 'The Silver Chair']))->commentable()->associate($post))->save();
+
+        $this->assertInstanceOf(MorphOne::class, $post->comments()->one());
+
+        $this->assertEquals($latestComment->id, $post->latestComment->id);
+        $this->assertEquals($oldestComment->id, $post->oldestComment->id);
+    }
 }
 
 class Post extends Model
@@ -64,6 +82,16 @@ class Post extends Model
     {
         return $this->morphMany(Comment::class, 'commentable');
     }
+
+    public function latestComment(): MorphOne
+    {
+        return $this->comments()->one()->latestOfMany();
+    }
+
+    public function oldestComment(): MorphOne
+    {
+        return $this->comments()->one()->oldestOfMany();
+    }
 }
 
 class Comment extends Model
