@@ -8,7 +8,7 @@ use Illuminate\Support\Facades\Schema;
 
 class EloquentPaginateTest extends DatabaseTestCase
 {
-    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
+    protected function afterRefreshingDatabase()
     {
         Schema::create('posts', function (Blueprint $table) {
             $table->increments('id');
