@@ -3,19 +3,22 @@
 namespace Illuminate\Tests\Integration\Foundation\Testing\Concerns;
 
 use Illuminate\Database\Schema\Blueprint;
-use Illuminate\Foundation\Auth\User as Authenticatable;
+use Illuminate\Foundation\Auth\User;
+use Illuminate\Foundation\Testing\RefreshDatabase;
 use Illuminate\Http\Request;
 use Illuminate\Support\Facades\Auth;
 use Illuminate\Support\Facades\Route;
 use Illuminate\Support\Facades\Schema;
+use Orchestra\Testbench\Attributes\WithMigration;
 use Orchestra\Testbench\TestCase;
 
+#[WithMigration]
 class InteractsWithAuthenticationTest extends TestCase
 {
-    protected function getEnvironmentSetUp($app)
-    {
-        $app['config']->set('auth.providers.users.model', AuthenticationTestUser::class);
+    use RefreshDatabase;
 
+    protected function defineEnvironment($app)
+    {
         $app['config']->set('auth.guards.api', [
             'driver' => 'token',
             'provider' => 'users',
@@ -23,20 +26,17 @@ class InteractsWithAuthenticationTest extends TestCase
         ]);
     }
 
-    protected function setUp(): void
+    protected function afterRefreshingDatabase()
     {
-        parent::setUp();
+        Schema::table('users', function (Blueprint $table) {
+            $table->renameColumn('name', 'username');
+        });
 
-        Schema::create('users', function (Blueprint $table) {
-            $table->increments('id');
-            $table->string('email');
-            $table->string('username');
-            $table->string('password');
-            $table->string('remember_token')->default(null)->nullable();
+        Schema::table('users', function (Blueprint $table) {
             $table->tinyInteger('is_active')->default(0);
         });
 
-        AuthenticationTestUser::create([
+        User::forceCreate([
             'username' => 'taylorotwell',
             'email' => 'taylorotwell@laravel.com',
             'password' => bcrypt('password'),
@@ -50,7 +50,7 @@ class InteractsWithAuthenticationTest extends TestCase
             return 'Hello '.$request->user()->username;
         })->middleware(['auth']);
 
-        $user = AuthenticationTestUser::where('username', '=', 'taylorotwell')->first();
+        $user = User::where('username', '=', 'taylorotwell')->first();
 
         $this->actingAs($user)
             ->get('/me')
@@ -68,7 +68,7 @@ class InteractsWithAuthenticationTest extends TestCase
             return $request->user();
         });
 
-        $user = AuthenticationTestUser::where('username', '=', 'taylorotwell')->first();
+        $user = User::where('username', '=', 'taylorotwell')->first();
 
         $this->actingAs($user, 'api')
             ->get('/me')
@@ -76,25 +76,3 @@ class InteractsWithAuthenticationTest extends TestCase
             ->assertSeeText('Hello taylorotwell');
     }
 }
-
-class AuthenticationTestUser extends Authenticatable
-{
-    public $table = 'users';
-    public $timestamps = false;
-
-    /**
-     * The attributes that are mass assignable.
-     *
-     * @var string[]
-     */
-    protected $guarded = [];
-
-    /**
-     * The attributes that should be hidden for arrays.
-     *
-     * @var string[]
-     */
-    protected $hidden = [
-        'password', 'remember_token',
-    ];
-}
