@@ -6,6 +6,7 @@ use Illuminate\Database\Eloquent\Collection;
 use Illuminate\Database\Eloquent\Model;
 use Illuminate\Database\Eloquent\Relations\Pivot;
 use Illuminate\Database\Schema\Blueprint;
+use Illuminate\Queue\Attributes\WithoutRelations;
 use Illuminate\Queue\SerializesModels;
 use LogicException;
 use Orchestra\Testbench\TestCase;
@@ -274,9 +275,6 @@ class ModelSerializationTest extends TestCase
         $this->assertInstanceOf(ModelSerializationTestCustomUserCollection::class, $unserialized->users);
     }
 
-    /**
-     * @requires PHP >= 7.4
-     */
     public function testItSerializesTypedProperties()
     {
         require_once __DIR__.'/typed-properties.php';
@@ -317,9 +315,54 @@ class ModelSerializationTest extends TestCase
         $serialized = serialize(new ModelSerializationParentAccessibleTestClass($user, $user, $user));
 
         $this->assertSame(
-            'O:78:"Illuminate\\Tests\\Integration\\Queue\\ModelSerializationParentAccessibleTestClass":2:{s:4:"user";O:45:"Illuminate\\Contracts\\Database\\ModelIdentifier":4:{s:5:"class";s:61:"Illuminate\\Tests\\Integration\\Queue\\ModelSerializationTestUser";s:2:"id";i:1;s:9:"relations";a:0:{}s:10:"connection";s:7:"testing";}s:8:"'."\0".'*'."\0".'user2";O:45:"Illuminate\\Contracts\\Database\\ModelIdentifier":4:{s:5:"class";s:61:"Illuminate\\Tests\\Integration\\Queue\\ModelSerializationTestUser";s:2:"id";i:1;s:9:"relations";a:0:{}s:10:"connection";s:7:"testing";}}', $serialized
+            'O:78:"Illuminate\\Tests\\Integration\\Queue\\ModelSerializationParentAccessibleTestClass":2:{s:4:"user";O:45:"Illuminate\\Contracts\\Database\\ModelIdentifier":5:{s:5:"class";s:61:"Illuminate\\Tests\\Integration\\Queue\\ModelSerializationTestUser";s:2:"id";i:1;s:9:"relations";a:0:{}s:10:"connection";s:7:"testing";s:15:"collectionClass";N;}s:8:"'."\0".'*'."\0".'user2";O:45:"Illuminate\\Contracts\\Database\\ModelIdentifier":5:{s:5:"class";s:61:"Illuminate\\Tests\\Integration\\Queue\\ModelSerializationTestUser";s:2:"id";i:1;s:9:"relations";a:0:{}s:10:"connection";s:7:"testing";s:15:"collectionClass";N;}}', $serialized
         );
     }
+
+    public function test_it_respects_without_relations_attribute()
+    {
+        $user = User::create([
+            'email' => 'taylor@laravel.com',
+        ])->load(['roles']);
+
+        $serialized = serialize(new ModelSerializationWithoutRelations($user));
+
+        $this->assertSame(
+            'O:69:"Illuminate\Tests\Integration\Queue\ModelSerializationWithoutRelations":1:{s:4:"user";O:45:"Illuminate\Contracts\Database\ModelIdentifier":5:{s:5:"class";s:39:"Illuminate\Tests\Integration\Queue\User";s:2:"id";i:1;s:9:"relations";a:0:{}s:10:"connection";s:7:"testing";s:15:"collectionClass";N;}}', $serialized
+        );
+    }
+
+    public function test_it_respects_without_relations_attribute_applied_to_class()
+    {
+        $user = User::create([
+            'email' => 'taylor@laravel.com',
+        ])->load(['roles']);
+
+        $serialized = serialize(new ModelSerializationAttributeTargetsClassTestClass($user, new DataValueObject('hello')));
+
+        $this->assertSame(
+            'O:83:"Illuminate\Tests\Integration\Queue\ModelSerializationAttributeTargetsClassTestClass":2:{s:4:"user";O:45:"Illuminate\Contracts\Database\ModelIdentifier":5:{s:5:"class";s:39:"Illuminate\Tests\Integration\Queue\User";s:2:"id";i:1;s:9:"relations";a:0:{}s:10:"connection";s:7:"testing";s:15:"collectionClass";N;}s:5:"value";O:50:"Illuminate\Tests\Integration\Queue\DataValueObject":1:{s:5:"value";s:5:"hello";}}',
+            $serialized
+        );
+
+        /** @var ModelSerializationAttributeTargetsClassTestClass $unserialized */
+        $unserialized = unserialize($serialized);
+
+        $this->assertFalse($unserialized->user->relationLoaded('roles'));
+        $this->assertEquals('hello', $unserialized->value->value);
+    }
+
+    public function test_serialization_types_empty_custom_eloquent_collection()
+    {
+        $class = new ModelSerializationTypedCustomCollectionTestClass(
+            new ModelSerializationTestCustomUserCollection());
+
+        $serialized = serialize($class);
+
+        unserialize($serialized);
+
+        $this->assertTrue(true);
+    }
 }
 
 trait TraitBootsAndInitializersTest
@@ -355,6 +398,18 @@ class ModelSerializationTestCustomUserCollection extends Collection
     //
 }
 
+class ModelSerializationTypedCustomCollectionTestClass
+{
+    use SerializesModels;
+
+    public ModelSerializationTestCustomUserCollection $collection;
+
+    public function __construct(ModelSerializationTestCustomUserCollection $collection)
+    {
+        $this->collection = $collection;
+    }
+}
+
 class ModelSerializationTestCustomUser extends Model
 {
     public $table = 'users';
@@ -478,6 +533,29 @@ class ModelSerializationParentAccessibleTestClass extends ModelSerializationAcce
     //
 }
 
+class ModelSerializationWithoutRelations
+{
+    use SerializesModels;
+
+    #[WithoutRelations]
+    public User $user;
+
+    public function __construct(User $user)
+    {
+        $this->user = $user;
+    }
+}
+
+#[WithoutRelations]
+class ModelSerializationAttributeTargetsClassTestClass
+{
+    use SerializesModels;
+
+    public function __construct(public User $user, public DataValueObject $value)
+    {
+    }
+}
+
 class ModelRelationSerializationTestClass
 {
     use SerializesModels;
@@ -501,3 +579,10 @@ class CollectionSerializationTestClass
         $this->users = $users;
     }
 }
+
+class DataValueObject
+{
+    public function __construct(public $value = 1)
+    {
+    }
+}
