@@ -2,6 +2,8 @@
 
 namespace Illuminate\Tests\Integration\Routing;
 
+use Illuminate\Foundation\Auth\User;
+use Illuminate\Routing\Middleware\SubstituteBindings;
 use Illuminate\Support\Facades\Route;
 use Orchestra\Testbench\TestCase;
 
@@ -20,7 +22,7 @@ class RouteRedirectTest extends TestCase
         $response->assertStatus(301);
     }
 
-    public function routeRedirectDataSets()
+    public static function routeRedirectDataSets()
     {
         return [
             'route redirect with no parameters' => ['from', 'to', '/from', '/to'],
@@ -34,4 +36,42 @@ class RouteRedirectTest extends TestCase
             'route redirect with two optional replacements that switch position' => ['users/{user?}/{switch?}', 'members/{switch?}/{user?}', '/users/11/22', '/members/22/11'],
         ];
     }
+
+    public function testRouteRedirectWithExplicitRouteModelBinding()
+    {
+        $this->withoutExceptionHandling();
+        Route::middleware([SubstituteBindings::class])->group(function () {
+            Route::redirect('users/{user}', 'users/{user}/overview');
+        });
+        Route::bind('user', fn ($id) => (new User())->setAttribute('id', '999'));
+
+        $response = $this->get('users/1');
+
+        $response->assertRedirect('users/999/overview');
+    }
+
+    public function testToRouteHelper()
+    {
+        Route::get('to', function () {
+            // ..
+        })->name('to');
+
+        Route::get('from-301', function () {
+            return to_route('to', [], 301);
+        });
+
+        Route::get('from-302', function () {
+            return to_route('to');
+        });
+
+        $this->get('from-301')
+            ->assertRedirect('to')
+            ->assertStatus(301)
+            ->assertSee('Redirecting to');
+
+        $this->get('from-302')
+            ->assertRedirect('to')
+            ->assertStatus(302)
+            ->assertSee('Redirecting to');
+    }
 }
