@@ -2,18 +2,30 @@
 
 namespace Illuminate\Tests\Mail;
 
+use Aws\Command;
+use Aws\Exception\AwsException;
 use Aws\Ses\SesClient;
 use Illuminate\Config\Repository;
 use Illuminate\Container\Container;
 use Illuminate\Mail\MailManager;
 use Illuminate\Mail\Transport\SesTransport;
-use Illuminate\Support\Str;
 use Illuminate\View\Factory;
+use Mockery as m;
 use PHPUnit\Framework\TestCase;
-use Swift_Message;
+use Symfony\Component\Mailer\Exception\TransportException;
+use Symfony\Component\Mailer\Header\MetadataHeader;
+use Symfony\Component\Mime\Address;
+use Symfony\Component\Mime\Email;
 
 class MailSesTransportTest extends TestCase
 {
+    protected function tearDown(): void
+    {
+        m::close();
+
+        parent::tearDown();
+    }
+
     public function testGetTransport()
     {
         $container = new Container;
@@ -31,42 +43,59 @@ class MailSesTransportTest extends TestCase
         $manager = new MailManager($container);
 
         /** @var \Illuminate\Mail\Transport\SesTransport $transport */
-        $transport = $manager->createTransport(['transport' => 'ses']);
+        $transport = $manager->createSymfonyTransport(['transport' => 'ses']);
 
         $ses = $transport->ses();
 
         $this->assertSame('us-east-1', $ses->getRegion());
+
+        $this->assertSame('ses', (string) $transport);
     }
 
     public function testSend()
     {
-        $message = new Swift_Message('Foo subject', 'Bar body');
-        $message->setSender('myself@example.com');
-        $message->setTo('me@example.com');
-        $message->setBcc('you@example.com');
+        $message = new Email();
+        $message->subject('Foo subject');
+        $message->text('Bar body');
+        $message->sender('myself@example.com');
+        $message->to('me@example.com');
+        $message->bcc('you@example.com');
+        $message->replyTo(new Address('taylor@example.com', 'Taylor Otwell'));
+        $message->getHeaders()->add(new MetadataHeader('FooTag', 'TagValue'));
 
-        $client = $this->getMockBuilder(SesClient::class)
-            ->addMethods(['sendRawEmail'])
-            ->disableOriginalConstructor()
-            ->getMock();
-        $transport = new SesTransport($client);
+        $client = m::mock(SesClient::class);
+        $sesResult = m::mock();
+        $sesResult->shouldReceive('get')
+            ->with('MessageId')
+            ->once()
+            ->andReturn('ses-message-id');
+        $client->shouldReceive('sendRawEmail')->once()
+            ->with(m::on(function ($arg) {
+                return $arg['Source'] === 'myself@example.com' &&
+                    $arg['Destinations'] === ['me@example.com', 'you@example.com'] &&
+                    $arg['Tags'] === [['Name' => 'FooTag', 'Value' => 'TagValue']] &&
+                    strpos($arg['RawMessage']['Data'], 'Reply-To: Taylor Otwell <taylor@example.com>') !== false;
+            }))
+            ->andReturn($sesResult);
 
-        // Generate a messageId for our mock to return to ensure that the post-sent message
-        // has X-Message-ID in its headers
-        $messageId = Str::random(32);
-        $sendRawEmailMock = new SendRawEmailMock($messageId);
-        $client->expects($this->once())
-            ->method('sendRawEmail')
-            ->with($this->equalTo([
-                'Source' => 'myself@example.com',
-                'RawMessage' => ['Data' => (string) $message],
-            ]))
-            ->willReturn($sendRawEmailMock);
+        (new SesTransport($client))->send($message);
+    }
 
-        $transport->send($message);
+    public function testSendError()
+    {
+        $message = new Email();
+        $message->subject('Foo subject');
+        $message->text('Bar body');
+        $message->sender('myself@example.com');
+        $message->to('me@example.com');
 
-        $this->assertEquals($messageId, $message->getHeaders()->get('X-Message-ID')->getFieldBody());
-        $this->assertEquals($messageId, $message->getHeaders()->get('X-SES-Message-ID')->getFieldBody());
+        $client = m::mock(SesClient::class);
+        $client->shouldReceive('sendRawEmail')->once()
+            ->andThrow(new AwsException('Email address is not verified.', new Command('sendRawEmail')));
+
+        $this->expectException(TransportException::class);
+
+        (new SesTransport($client))->send($message);
     }
 
     public function testSesLocalConfiguration()
@@ -109,7 +138,7 @@ class MailSesTransportTest extends TestCase
         $mailer = $manager->mailer('ses');
 
         /** @var \Illuminate\Mail\Transport\SesTransport $transport */
-        $transport = $mailer->getSwiftMailer()->getTransport();
+        $transport = $mailer->getSymfonyTransport();
 
         $this->assertSame('eu-west-1', $transport->ses()->getRegion());
 
@@ -121,18 +150,3 @@ class MailSesTransportTest extends TestCase
         ], $transport->getOptions());
     }
 }
-
-class SendRawEmailMock
-{
-    protected $getResponse;
-
-    public function __construct($responseValue)
-    {
-        $this->getResponse = $responseValue;
-    }
-
-    public function get($key)
-    {
-        return $this->getResponse;
-    }
-}
