@@ -2,6 +2,8 @@
 
 namespace Illuminate\Tests\Notifications;
 
+use Illuminate\Contracts\Mail\Attachable;
+use Illuminate\Mail\Attachment;
 use Illuminate\Notifications\Messages\MailMessage;
 use PHPUnit\Framework\TestCase;
 
@@ -121,6 +123,26 @@ class NotificationMailMessageTest extends TestCase
         $this->assertSame([['test@example.com', null], ['test@example.com', 'Test']], $message->replyTo);
     }
 
+    public function testMetadataIsSetCorrectly()
+    {
+        $message = new MailMessage;
+        $message->metadata('origin', 'test-suite');
+        $message->metadata('user_id', 1);
+
+        $this->assertArrayHasKey('origin', $message->metadata);
+        $this->assertSame('test-suite', $message->metadata['origin']);
+        $this->assertArrayHasKey('user_id', $message->metadata);
+        $this->assertSame(1, $message->metadata['user_id']);
+    }
+
+    public function testTagIsSetCorrectly()
+    {
+        $message = new MailMessage;
+        $message->tag('test');
+
+        $this->assertContains('test', $message->tags);
+    }
+
     public function testCallbackIsSetCorrectly()
     {
         $callback = function () {
@@ -128,7 +150,7 @@ class NotificationMailMessageTest extends TestCase
         };
 
         $message = new MailMessage;
-        $message->withSwiftMessage($callback);
+        $message->withSymfonyMessage($callback);
 
         $this->assertSame([$callback], $message->callbacks);
     }
@@ -250,4 +272,91 @@ class NotificationMailMessageTest extends TestCase
         $message->unless('truthy', $callback, $default);
         $this->assertSame([['truthy@example.com', null]], $message->cc);
     }
+
+    public function testItAttachesFilesViaAttachableContractFromPath()
+    {
+        $message = new MailMessage;
+
+        $message->attach(new class() implements Attachable
+        {
+            public function toMailAttachment()
+            {
+                return Attachment::fromPath('/foo.jpg')->as('bar')->withMime('image/png');
+            }
+        });
+
+        $this->assertSame([
+            'file' => '/foo.jpg',
+            'options' => [
+                'as' => 'bar',
+                'mime' => 'image/png',
+            ],
+        ], $message->attachments[0]);
+    }
+
+    public function testItAttachesFilesViaAttachableContractFromData()
+    {
+        $mailMessage = new MailMessage();
+
+        $mailMessage->attach(new class() implements Attachable
+        {
+            public function toMailAttachment()
+            {
+                return Attachment::fromData(fn () => 'bar', 'foo.jpg')->withMime('image/png');
+            }
+        });
+
+        $this->assertSame([
+            'data' => 'bar',
+            'name' => 'foo.jpg',
+            'options' => [
+                'mime' => 'image/png',
+            ],
+        ], $mailMessage->rawAttachments[0]);
+    }
+
+    public function testItAttachesManyFiles()
+    {
+        $mailMessage = new MailMessage();
+        $attachable = new class() implements Attachable
+        {
+            public function toMailAttachment()
+            {
+                return Attachment::fromData(fn () => 'bar', 'foo.jpg')->withMime('image/png');
+            }
+        };
+
+        $mailMessage->attachMany([
+            $attachable,
+            '/path/to/forge.svg',
+            '/path/to/vapor.svg' => [
+                'as' => 'Logo.svg',
+                'mime' => 'image/svg+xml',
+            ],
+        ]);
+
+        $this->assertSame([
+            [
+                'data' => 'bar',
+                'name' => 'foo.jpg',
+                'options' => [
+                    'mime' => 'image/png',
+                ],
+            ],
+        ], $mailMessage->rawAttachments);
+
+        $this->assertSame([
+            [
+                'file' => '/path/to/forge.svg',
+                'options' => [],
+            ],
+            [
+                'file' => '/path/to/vapor.svg',
+                'options' => [
+                    'as' => 'Logo.svg',
+                    'mime' => 'image/svg+xml',
+                ],
+            ],
+        ], $mailMessage->attachments);
+    }
 }
