@@ -74,17 +74,25 @@ class LengthAwarePaginatorTest extends TestCase
         $this->p->setPath('http://website.com');
         $this->p->setPageName('foo');
 
-        $this->assertSame('http://website.com',
-                            $this->p->path());
+        $this->assertSame(
+            'http://website.com',
+            $this->p->path()
+        );
 
-        $this->assertSame('http://website.com?foo=2',
-                            $this->p->url($this->p->currentPage()));
+        $this->assertSame(
+            'http://website.com?foo=2',
+            $this->p->url($this->p->currentPage())
+        );
 
-        $this->assertSame('http://website.com?foo=1',
-                            $this->p->url($this->p->currentPage() - 1));
+        $this->assertSame(
+            'http://website.com?foo=1',
+            $this->p->url($this->p->currentPage() - 1)
+        );
 
-        $this->assertSame('http://website.com?foo=1',
-                            $this->p->url($this->p->currentPage() - 2));
+        $this->assertSame(
+            'http://website.com?foo=1',
+            $this->p->url($this->p->currentPage() - 2)
+        );
     }
 
     public function testLengthAwarePaginatorCanGenerateUrlsWithQuery()
@@ -92,8 +100,10 @@ class LengthAwarePaginatorTest extends TestCase
         $this->p->setPath('http://website.com?sort_by=date');
         $this->p->setPageName('foo');
 
-        $this->assertSame('http://website.com?sort_by=date&foo=2',
-                            $this->p->url($this->p->currentPage()));
+        $this->assertSame(
+            'http://website.com?sort_by=date&foo=2',
+            $this->p->url($this->p->currentPage())
+        );
     }
 
     public function testLengthAwarePaginatorCanGenerateUrlsWithoutTrailingSlashes()
@@ -101,14 +111,20 @@ class LengthAwarePaginatorTest extends TestCase
         $this->p->setPath('http://website.com/test');
         $this->p->setPageName('foo');
 
-        $this->assertSame('http://website.com/test?foo=2',
-                            $this->p->url($this->p->currentPage()));
+        $this->assertSame(
+            'http://website.com/test?foo=2',
+            $this->p->url($this->p->currentPage())
+        );
 
-        $this->assertSame('http://website.com/test?foo=1',
-                            $this->p->url($this->p->currentPage() - 1));
+        $this->assertSame(
+            'http://website.com/test?foo=1',
+            $this->p->url($this->p->currentPage() - 1)
+        );
 
-        $this->assertSame('http://website.com/test?foo=1',
-                            $this->p->url($this->p->currentPage() - 2));
+        $this->assertSame(
+            'http://website.com/test?foo=1',
+            $this->p->url($this->p->currentPage() - 2)
+        );
     }
 
     public function testLengthAwarePaginatorCorrectlyGenerateUrlsWithQueryAndSpaces()
@@ -116,8 +132,10 @@ class LengthAwarePaginatorTest extends TestCase
         $this->p->setPath('http://website.com?key=value%20with%20spaces');
         $this->p->setPageName('foo');
 
-        $this->assertSame('http://website.com?key=value%20with%20spaces&foo=2',
-                            $this->p->url($this->p->currentPage()));
+        $this->assertSame(
+            'http://website.com?key=value%20with%20spaces&foo=2',
+            $this->p->url($this->p->currentPage())
+        );
     }
 
     public function testItRetrievesThePaginatorOptions()
