@@ -6,6 +6,7 @@ use Illuminate\Container\Container;
 use Illuminate\Pipeline\Pipeline;
 use PHPUnit\Framework\TestCase;
 use RuntimeException;
+use stdClass;
 
 class PipelineTest extends TestCase
 {
@@ -96,6 +97,32 @@ class PipelineTest extends TestCase
         unset($_SERVER['__test.pipe.one']);
     }
 
+    public function testPipelineUsageWithPipe()
+    {
+        $object = new stdClass();
+
+        $object->value = 0;
+
+        $function = function ($object, $next) {
+            $object->value++;
+
+            return $next($object);
+        };
+
+        $result = (new Pipeline(new Container))
+            ->send($object)
+            ->through([$function])
+            ->pipe([$function])
+            ->then(
+                function ($piped) {
+                    return $piped;
+                }
+            );
+
+        $this->assertSame($object, $result);
+        $this->assertEquals(2, $object->value);
+    }
+
     public function testPipelineUsageWithInvokableClass()
     {
         $result = (new Pipeline(new Container))
@@ -113,6 +140,56 @@ class PipelineTest extends TestCase
         unset($_SERVER['__test.pipe.one']);
     }
 
+    public function testThenMethodIsNotCalledIfThePipeReturns()
+    {
+        $_SERVER['__test.pipe.then'] = '(*_*)';
+        $_SERVER['__test.pipe.second'] = '(*_*)';
+
+        $result = (new Pipeline(new Container))
+            ->send('foo')
+            ->through([
+                fn ($value, $next) => 'm(-_-)m',
+                fn ($value, $next) => $_SERVER['__test.pipe.second'] = 'm(-_-)m',
+            ])
+            ->then(function ($piped) {
+                $_SERVER['__test.pipe.then'] = '(0_0)';
+
+                return $piped;
+            });
+
+        $this->assertSame('m(-_-)m', $result);
+        // The then callback is not called.
+        $this->assertSame('(*_*)', $_SERVER['__test.pipe.then']);
+        // The second pipe is not called.
+        $this->assertSame('(*_*)', $_SERVER['__test.pipe.second']);
+
+        unset($_SERVER['__test.pipe.then']);
+    }
+
+    public function testThenMethodInputValue()
+    {
+        $result = (new Pipeline(new Container))
+            ->send('foo')
+            ->through([function ($value, $next) {
+                $value = $next('::not_foo::');
+
+                $_SERVER['__test.pipe.return'] = $value;
+
+                return 'pipe::'.$value;
+            }])
+            ->then(function ($piped) {
+                $_SERVER['__test.then.arg'] = $piped;
+
+                return 'then'.$piped;
+            });
+
+        $this->assertSame('pipe::then::not_foo::', $result);
+        $this->assertSame('::not_foo::', $_SERVER['__test.then.arg']);
+
+        unset($_SERVER['__test.then.arg']);
+        unset($_SERVER['__test.pipe.return']);
+    }
+
     public function testPipelineUsageWithParameters()
     {
         $parameters = ['one', 'two'];
@@ -166,6 +243,36 @@ class PipelineTest extends TestCase
 
         unset($_SERVER['__test.pipe.one']);
     }
+
+    public function testPipelineConditionable()
+    {
+        $result = (new Pipeline(new Container))
+            ->send('foo')
+            ->when(true, function (Pipeline $pipeline) {
+                $pipeline->pipe([PipelineTestPipeOne::class]);
+            })
+            ->then(function ($piped) {
+                return $piped;
+            });
+
+        $this->assertSame('foo', $result);
+        $this->assertSame('foo', $_SERVER['__test.pipe.one']);
+        unset($_SERVER['__test.pipe.one']);
+
+        $_SERVER['__test.pipe.one'] = null;
+        $result = (new Pipeline(new Container))
+            ->send('foo')
+            ->when(false, function (Pipeline $pipeline) {
+                $pipeline->pipe([PipelineTestPipeOne::class]);
+            })
+            ->then(function ($piped) {
+                return $piped;
+            });
+
+        $this->assertSame('foo', $result);
+        $this->assertNull($_SERVER['__test.pipe.one']);
+        unset($_SERVER['__test.pipe.one']);
+    }
 }
 
 class PipelineTestPipeOne
