@@ -19,10 +19,11 @@ class QueueDatabaseQueueUnitTest extends TestCase
         m::close();
     }
 
-    public function testPushProperlyPushesJobOntoDatabase()
+    /**
+     * @dataProvider pushJobsDataProvider
+     */
+    public function testPushProperlyPushesJobOntoDatabase($uuid, $job, $displayNameStartsWith, $jobStartsWith)
     {
-        $uuid = Str::uuid();
-
         Str::createUuidsUsing(function () use ($uuid) {
             return $uuid;
         });
@@ -31,21 +32,36 @@ class QueueDatabaseQueueUnitTest extends TestCase
         $queue->expects($this->any())->method('currentTime')->willReturn('time');
         $queue->setContainer($container = m::spy(Container::class));
         $database->shouldReceive('table')->with('table')->andReturn($query = m::mock(stdClass::class));
-        $query->shouldReceive('insertGetId')->once()->andReturnUsing(function ($array) use ($uuid) {
+        $query->shouldReceive('insertGetId')->once()->andReturnUsing(function ($array) use ($uuid, $displayNameStartsWith, $jobStartsWith) {
+            $payload = json_decode($array['payload'], true);
+            $this->assertSame($uuid, $payload['uuid']);
+            $this->assertStringContainsString($displayNameStartsWith, $payload['displayName']);
+            $this->assertStringContainsString($jobStartsWith, $payload['job']);
+
             $this->assertSame('default', $array['queue']);
-            $this->assertSame(json_encode(['uuid' => $uuid, 'displayName' => 'foo', 'job' => 'foo', 'maxTries' => null, 'maxExceptions' => null, 'failOnTimeout' => false, 'backoff' => null, 'timeout' => null, 'data' => ['data']]), $array['payload']);
             $this->assertEquals(0, $array['attempts']);
             $this->assertNull($array['reserved_at']);
             $this->assertIsInt($array['available_at']);
         });
 
-        $queue->push('foo', ['data']);
+        $queue->push($job, ['data']);
 
-        $container->shouldHaveReceived('bound')->with('events')->once();
+        $container->shouldHaveReceived('bound')->with('events')->twice();
 
         Str::createUuidsNormally();
     }
 
+    public static function pushJobsDataProvider()
+    {
+        $uuid = Str::uuid()->toString();
+
+        return [
+            [$uuid, new MyTestJob, 'MyTestJob', 'CallQueuedHandler'],
+            [$uuid, fn () => 0, 'Closure', 'CallQueuedHandler'],
+            [$uuid, 'foo', 'foo', 'foo'],
+        ];
+    }
+
     public function testDelayedPushProperlyPushesJobOntoDatabase()
     {
         $uuid = Str::uuid();
@@ -54,11 +70,10 @@ class QueueDatabaseQueueUnitTest extends TestCase
             return $uuid;
         });
 
-        $queue = $this->getMockBuilder(
-            DatabaseQueue::class)->onlyMethods(
-            ['currentTime'])->setConstructorArgs(
-            [$database = m::mock(Connection::class), 'table', 'default']
-        )->getMock();
+        $queue = $this->getMockBuilder(DatabaseQueue::class)
+            ->onlyMethods(['currentTime'])
+            ->setConstructorArgs([$database = m::mock(Connection::class), 'table', 'default'])
+            ->getMock();
         $queue->expects($this->any())->method('currentTime')->willReturn('time');
         $queue->setContainer($container = m::spy(Container::class));
         $database->shouldReceive('table')->with('table')->andReturn($query = m::mock(stdClass::class));
@@ -72,7 +87,7 @@ class QueueDatabaseQueueUnitTest extends TestCase
 
         $queue->later(10, 'foo', ['data']);
 
-        $container->shouldHaveReceived('bound')->with('events')->once();
+        $container->shouldHaveReceived('bound')->with('events')->twice();
 
         Str::createUuidsNormally();
     }
@@ -84,11 +99,10 @@ class QueueDatabaseQueueUnitTest extends TestCase
         $job = new stdClass;
         $job->invalid = "\xc3\x28";
 
-        $queue = $this->getMockForAbstractClass(Queue::class);
+        $queue = m::mock(Queue::class)->makePartial();
         $class = new ReflectionClass(Queue::class);
 
         $createPayload = $class->getMethod('createPayload');
-        $createPayload->setAccessible(true);
         $createPayload->invokeArgs($queue, [
             $job,
             'queue-name',
@@ -99,11 +113,10 @@ class QueueDatabaseQueueUnitTest extends TestCase
     {
         $this->expectException('InvalidArgumentException');
 
-        $queue = $this->getMockForAbstractClass(Queue::class);
+        $queue = m::mock(Queue::class)->makePartial();
         $class = new ReflectionClass(Queue::class);
 
         $createPayload = $class->getMethod('createPayload');
-        $createPayload->setAccessible(true);
         $createPayload->invokeArgs($queue, [
             ["\xc3\x28"],
             'queue-name',
@@ -154,3 +167,11 @@ class QueueDatabaseQueueUnitTest extends TestCase
         $this->assertArrayHasKey('payload', array_slice($record, -1, 1, true));
     }
 }
+
+class MyTestJob
+{
+    public function handle()
+    {
+        // ...
+    }
+}
