@@ -51,7 +51,7 @@ class QueueSyncQueueTest extends TestCase
 
         try {
             $sync->push(FailingSyncQueueTestHandler::class, ['foo' => 'bar']);
-        } catch (Exception $e) {
+        } catch (Exception) {
             $this->assertTrue($_SERVER['__sync.failed']);
         }
 
@@ -74,7 +74,7 @@ class QueueSyncQueueTest extends TestCase
         try {
             $sync->push(new SyncQueueJob());
         } catch (LogicException $e) {
-            $this->assertEquals('extraValue', $e->getMessage());
+            $this->assertSame('extraValue', $e->getMessage());
         }
     }
 }
