@@ -6,6 +6,7 @@ use Illuminate\Contracts\Events\Dispatcher;
 use Illuminate\Foundation\Application;
 use Illuminate\Foundation\Testing\Concerns\InteractsWithRedis;
 use Illuminate\Redis\Connections\Connection;
+use Illuminate\Redis\Connections\PhpRedisConnection;
 use Illuminate\Redis\RedisManager;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
@@ -329,6 +330,10 @@ class RedisConnectionTest extends TestCase
                     'count' => 2,
                 ],
             ]));
+            $this->assertEquals(['matt' => 5, 'taylor' => 10], $redis->zrangebyscore('set', 0, 11, [
+                'withscores' => true,
+                'limit' => [1, 2],
+            ]));
 
             $redis->flushall();
         }
@@ -346,6 +351,10 @@ class RedisConnectionTest extends TestCase
                     'count' => 2,
                 ],
             ]));
+            $this->assertEquals(['matt' => 5, 'jeffrey' => 1], $redis->ZREVRANGEBYSCORE('set', 10, 0, [
+                'withscores' => true,
+                'limit' => [1, 2],
+            ]));
 
             $redis->flushall();
         }
@@ -458,10 +467,38 @@ class RedisConnectionTest extends TestCase
         }
     }
 
+    public function testItFlushes()
+    {
+        foreach ($this->connections() as $redis) {
+            $redis->set('name', 'Till');
+            $this->assertSame(1, $redis->exists('name'));
+
+            $redis->flushdb();
+            $this->assertSame(0, $redis->exists('name'));
+        }
+    }
+
+    public function testItFlushesAsynchronous()
+    {
+        foreach ($this->connections() as $redis) {
+            $redis->set('name', 'Till');
+            $this->assertSame(1, $redis->exists('name'));
+
+            $redis->flushdb('ASYNC');
+            $this->assertSame(0, $redis->exists('name'));
+        }
+    }
+
     public function testItRunsEval()
     {
         foreach ($this->connections() as $redis) {
-            $redis->eval('redis.call("set", KEYS[1], ARGV[1])', 1, 'name', 'mohamed');
+            if ($redis instanceof PhpRedisConnection) {
+                // User must decide what needs to be serialized and compressed.
+                $redis->eval('redis.call("set", KEYS[1], ARGV[1])', 1, 'name', ...$redis->pack(['mohamed']));
+            } else {
+                $redis->eval('redis.call("set", KEYS[1], ARGV[1])', 1, 'name', 'mohamed');
+            }
+
             $this->assertSame('mohamed', $redis->get('name'));
 
             $redis->flushall();
@@ -569,7 +606,7 @@ class RedisConnectionTest extends TestCase
                 }
 
                 foreach ($returnedKeys as $returnedKey) {
-                    $this->assertTrue(in_array($returnedKey, $initialKeys));
+                    $this->assertContains($returnedKey, $initialKeys);
                 }
             } while ($iterator > 0);
 
@@ -690,6 +727,33 @@ class RedisConnectionTest extends TestCase
         }
     }
 
+    public function testItSPopsForKeys()
+    {
+        foreach ($this->connections() as $redis) {
+            $members = ['test:spop:1', 'test:spop:2', 'test:spop:3', 'test:spop:4'];
+
+            foreach ($members as $member) {
+                $redis->sadd('set', $member);
+            }
+
+            $result = $redis->spop('set');
+            $this->assertIsNotArray($result);
+            $this->assertContains($result, $members);
+
+            $result = $redis->spop('set', 1);
+
+            $this->assertIsArray($result);
+            $this->assertCount(1, $result);
+
+            $result = $redis->spop('set', 2);
+
+            $this->assertIsArray($result);
+            $this->assertCount(2, $result);
+
+            $redis->flushAll();
+        }
+    }
+
     public function testPhpRedisScanOption()
     {
         foreach ($this->connections() as $redis) {
@@ -742,7 +806,7 @@ class RedisConnectionTest extends TestCase
         $host = env('REDIS_HOST', '127.0.0.1');
         $port = env('REDIS_PORT', 6379);
 
-        $prefixedPhpredis = new RedisManager(new Application, 'phpredis', [
+        $connections[] = (new RedisManager(new Application, 'phpredis', [
             'cluster' => false,
             'default' => [
                 'url' => "redis://user@$host:$port",
@@ -752,9 +816,9 @@ class RedisConnectionTest extends TestCase
                 'options' => ['prefix' => 'laravel:'],
                 'timeout' => 0.5,
             ],
-        ]);
+        ]))->connection();
 
-        $persistentPhpRedis = new RedisManager(new Application, 'phpredis', [
+        $connections['persistent'] = (new RedisManager(new Application, 'phpredis', [
             'cluster' => false,
             'default' => [
                 'host' => $host,
@@ -765,9 +829,9 @@ class RedisConnectionTest extends TestCase
                 'persistent' => true,
                 'persistent_id' => 'laravel',
             ],
-        ]);
+        ]))->connection();
 
-        $serializerPhpRedis = new RedisManager(new Application, 'phpredis', [
+        $connections[] = (new RedisManager(new Application, 'phpredis', [
             'cluster' => false,
             'default' => [
                 'host' => $host,
@@ -776,9 +840,9 @@ class RedisConnectionTest extends TestCase
                 'options' => ['serializer' => Redis::SERIALIZER_JSON],
                 'timeout' => 0.5,
             ],
-        ]);
+        ]))->connection();
 
-        $scanRetryPhpRedis = new RedisManager(new Application, 'phpredis', [
+        $connections[] = (new RedisManager(new Application, 'phpredis', [
             'cluster' => false,
             'default' => [
                 'host' => $host,
@@ -787,12 +851,145 @@ class RedisConnectionTest extends TestCase
                 'options' => ['scan' => Redis::SCAN_RETRY],
                 'timeout' => 0.5,
             ],
-        ]);
+        ]))->connection();
 
-        $connections[] = $prefixedPhpredis->connection();
-        $connections[] = $serializerPhpRedis->connection();
-        $connections[] = $scanRetryPhpRedis->connection();
-        $connections['persistent'] = $persistentPhpRedis->connection();
+        if (defined('Redis::COMPRESSION_LZF')) {
+            $connections['compression_lzf'] = (new RedisManager(new Application, 'phpredis', [
+                'cluster' => false,
+                'default' => [
+                    'host' => $host,
+                    'port' => $port,
+                    'database' => 9,
+                    'options' => [
+                        'compression' => Redis::COMPRESSION_LZF,
+                        'name' => 'compression_lzf',
+                    ],
+                    'timeout' => 0.5,
+                ],
+            ]))->connection();
+        }
+
+        if (defined('Redis::COMPRESSION_ZSTD')) {
+            $connections['compression_zstd'] = (new RedisManager(new Application, 'phpredis', [
+                'cluster' => false,
+                'default' => [
+                    'host' => $host,
+                    'port' => $port,
+                    'database' => 10,
+                    'options' => [
+                        'compression' => Redis::COMPRESSION_ZSTD,
+                        'name' => 'compression_zstd',
+                    ],
+                    'timeout' => 0.5,
+                ],
+            ]))->connection();
+
+            $connections['compression_zstd_default'] = (new RedisManager(new Application, 'phpredis', [
+                'cluster' => false,
+                'default' => [
+                    'host' => $host,
+                    'port' => $port,
+                    'database' => 11,
+                    'options' => [
+                        'compression' => Redis::COMPRESSION_ZSTD,
+                        'compression_level' => Redis::COMPRESSION_ZSTD_DEFAULT,
+                        'name' => 'compression_zstd_default',
+                    ],
+                    'timeout' => 0.5,
+                ],
+            ]))->connection();
+
+            $connections['compression_zstd_min'] = (new RedisManager(new Application, 'phpredis', [
+                'cluster' => false,
+                'default' => [
+                    'host' => $host,
+                    'port' => $port,
+                    'database' => 12,
+                    'options' => [
+                        'compression' => Redis::COMPRESSION_ZSTD,
+                        'compression_level' => Redis::COMPRESSION_ZSTD_MIN,
+                        'name' => 'compression_zstd_min',
+                    ],
+                    'timeout' => 0.5,
+                ],
+            ]))->connection();
+
+            $connections['compression_zstd_max'] = (new RedisManager(new Application, 'phpredis', [
+                'cluster' => false,
+                'default' => [
+                    'host' => $host,
+                    'port' => $port,
+                    'database' => 13,
+                    'options' => [
+                        'compression' => Redis::COMPRESSION_ZSTD,
+                        'compression_level' => Redis::COMPRESSION_ZSTD_MAX,
+                        'name' => 'compression_zstd_max',
+                    ],
+                    'timeout' => 0.5,
+                ],
+            ]))->connection();
+        }
+
+        if (defined('Redis::COMPRESSION_LZ4')) {
+            $connections['compression_lz4'] = (new RedisManager(new Application, 'phpredis', [
+                'cluster' => false,
+                'default' => [
+                    'host' => $host,
+                    'port' => $port,
+                    'database' => 14,
+                    'options' => [
+                        'compression' => Redis::COMPRESSION_LZ4,
+                        'name' => 'compression_lz4',
+                    ],
+                    'timeout' => 0.5,
+                ],
+            ]))->connection();
+
+            $connections['compression_lz4_default'] = (new RedisManager(new Application, 'phpredis', [
+                'cluster' => false,
+                'default' => [
+                    'host' => $host,
+                    'port' => $port,
+                    'database' => 15,
+                    'options' => [
+                        'compression' => Redis::COMPRESSION_LZ4,
+                        'compression_level' => 0,
+                        'name' => 'compression_lz4_default',
+                    ],
+                    'timeout' => 0.5,
+                ],
+            ]))->connection();
+
+            $connections['compression_lz4_min'] = (new RedisManager(new Application, 'phpredis', [
+                'cluster' => false,
+                'default' => [
+                    'host' => $host,
+                    'port' => $port,
+                    'database' => 16,
+                    'options' => [
+                        'compression' => Redis::COMPRESSION_LZ4,
+                        'compression_level' => 1,
+                        'name' => 'compression_lz4_min',
+                    ],
+                    'timeout' => 0.5,
+                ],
+            ]))->connection();
+
+            $connections['compression_lz4_max'] = (new RedisManager(new Application, 'phpredis', [
+                'cluster' => false,
+                'default' => [
+                    'host' => $host,
+                    'port' => $port,
+                    'database' => 17,
+                    'options' => [
+                        'compression' => Redis::COMPRESSION_LZ4,
+                        'compression_level' => 12,
+                        'name' => 'compression_lz4_max',
+                    ],
+                    'timeout' => 0.5,
+                ],
+            ]))->connection();
+        }
 
         return $connections;
     }
