@@ -7,7 +7,6 @@ use FooController;
 use Illuminate\Container\Container;
 use Illuminate\Contracts\Events\Dispatcher;
 use Illuminate\Http\Request;
-use Illuminate\Routing\Route;
 use Illuminate\Routing\Router;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
@@ -137,6 +136,51 @@ class RouteRegistrarTest extends TestCase
         $this->assertEquals(['one'], $this->getRoute()->excludedMiddleware());
     }
 
+    public function testGetRouteWithTrashed()
+    {
+        $route = $this->router->get('users', [RouteRegistrarControllerStub::class, 'index'])->withTrashed();
+
+        $this->assertTrue($route->allowsTrashedBindings());
+    }
+
+    public function testResourceWithTrashed()
+    {
+        $this->router->resource('users', RouteRegistrarControllerStub::class)
+            ->only(['index', 'destroy'])
+            ->withTrashed([
+                'index',
+                'destroy',
+            ]);
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertTrue($route->allowsTrashedBindings());
+        }
+    }
+
+    public function testFallbackRoute()
+    {
+        $route = $this->router->fallback(function () {
+            return 'milwad';
+        });
+
+        $this->assertTrue($route->isFallback);
+    }
+
+    public function testSetFallbackRoute()
+    {
+        $route = $this->router->fallback(function () {
+            return 'milwad';
+        });
+        $route->setFallback(false);
+
+        $this->assertFalse($route->isFallback);
+
+        $route->setFallback(true);
+
+        $this->assertTrue($route->isFallback);
+    }
+
     public function testCanRegisterGetRouteWithClosureAction()
     {
         $this->router->middleware('get-middleware')->get('users', function () {
@@ -431,12 +475,32 @@ class RouteRegistrarTest extends TestCase
         $this->seeResponse('hello', Request::create('bar/baz', 'GET'));
     }
 
+    public function testRouteGroupChaining()
+    {
+        $this->router
+            ->group([], function ($router) {
+                $router->get('foo', function () {
+                    return 'hello';
+                });
+            })
+            ->group([], function ($router) {
+                $router->get('bar', function () {
+                    return 'goodbye';
+                });
+            });
+
+        $routeCollection = $this->router->getRoutes();
+
+        $this->assertInstanceOf(\Illuminate\Routing\Route::class, $routeCollection->match(Request::create('foo', 'GET')));
+        $this->assertInstanceOf(\Illuminate\Routing\Route::class, $routeCollection->match(Request::create('bar', 'GET')));
+    }
+
     public function testRegisteringNonApprovedAttributesThrows()
     {
         $this->expectException(BadMethodCallException::class);
-        $this->expectExceptionMessage('Method Illuminate\Routing\RouteRegistrar::missing does not exist.');
+        $this->expectExceptionMessage('Method Illuminate\Routing\RouteRegistrar::unsupportedMethod does not exist.');
 
-        $this->router->domain('foo')->missing('bar')->group(function ($router) {
+        $this->router->domain('foo')->unsupportedMethod('bar')->group(function ($router) {
             //
         });
     }
@@ -573,6 +637,19 @@ class RouteRegistrarTest extends TestCase
         $this->assertTrue($this->router->getRoutes()->hasNamedRoute('users.destroy'));
     }
 
+    public function testCanLimitAndExcludeMethodsOnRegisteredResource()
+    {
+        $this->router->resource('users', RouteRegistrarControllerStub::class)
+                     ->only('index', 'show', 'destroy')
+                     ->except('destroy');
+
+        $this->assertCount(2, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('users.index'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('users.show'));
+        $this->assertFalse($this->router->getRoutes()->hasNamedRoute('users.destroy'));
+    }
+
     public function testCanSetShallowOptionOnRegisteredResource()
     {
         $this->router->resource('users.tasks', RouteRegistrarControllerStub::class)->shallow();
@@ -852,6 +929,155 @@ class RouteRegistrarTest extends TestCase
         }
     }
 
+    public function testWhereInRegistration()
+    {
+        $wheres = ['foo' => 'one|two', 'bar' => 'one|two'];
+
+        $this->router->get('/{foo}/{bar}')->whereIn(['foo', 'bar'], ['one', 'two']);
+        $this->router->get('/api/{bar}/{foo}')->whereIn(['bar', 'foo'], ['one', 'two']);
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testGroupWhereNumberRegistrationOnRouteRegistrar()
+    {
+        $wheres = ['foo' => '[0-9]+', 'bar' => '[0-9]+'];
+
+        $this->router->prefix('/{foo}/{bar}')->whereNumber(['foo', 'bar'])->group(function ($router) {
+            $router->get('/');
+        });
+
+        $this->router->prefix('/api/{bar}/{foo}')->whereNumber(['bar', 'foo'])->group(function ($router) {
+            $router->get('/');
+        });
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testGroupWhereAlphaRegistrationOnRouteRegistrar()
+    {
+        $wheres = ['foo' => '[a-zA-Z]+', 'bar' => '[a-zA-Z]+'];
+
+        $this->router->prefix('/{foo}/{bar}')->whereAlpha(['foo', 'bar'])->group(function ($router) {
+            $router->get('/');
+        });
+
+        $this->router->prefix('/api/{bar}/{foo}')->whereAlpha(['bar', 'foo'])->group(function ($router) {
+            $router->get('/');
+        });
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testGroupWhereAlphaNumericRegistrationOnRouteRegistrar()
+    {
+        $wheres = ['1a2b3c' => '[a-zA-Z0-9]+'];
+
+        $this->router->prefix('/{foo}')->whereAlphaNumeric(['1a2b3c'])->group(function ($router) {
+            $router->get('/');
+        });
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testGroupWhereInRegistrationOnRouteRegistrar()
+    {
+        $wheres = ['foo' => 'one|two', 'bar' => 'one|two'];
+
+        $this->router->prefix('/{foo}/{bar}')->whereIn(['foo', 'bar'], ['one', 'two'])->group(function ($router) {
+            $router->get('/');
+        });
+
+        $this->router->prefix('/api/{bar}/{foo}')->whereIn(['bar', 'foo'], ['one', 'two'])->group(function ($router) {
+            $router->get('/');
+        });
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testGroupWhereNumberRegistrationOnRouter()
+    {
+        $wheres = ['foo' => '[0-9]+', 'bar' => '[0-9]+'];
+
+        $this->router->whereNumber(['foo', 'bar'])->prefix('/{foo}/{bar}')->group(function ($router) {
+            $router->get('/');
+        });
+
+        $this->router->whereNumber(['bar', 'foo'])->prefix('/api/{bar}/{foo}')->group(function ($router) {
+            $router->get('/');
+        });
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testGroupWhereAlphaRegistrationOnRouter()
+    {
+        $wheres = ['foo' => '[a-zA-Z]+', 'bar' => '[a-zA-Z]+'];
+
+        $this->router->whereAlpha(['foo', 'bar'])->prefix('/{foo}/{bar}')->group(function ($router) {
+            $router->get('/');
+        });
+
+        $this->router->whereAlpha(['bar', 'foo'])->prefix('/api/{bar}/{foo}')->group(function ($router) {
+            $router->get('/');
+        });
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testGroupWhereAlphaNumericRegistrationOnRouter()
+    {
+        $wheres = ['1a2b3c' => '[a-zA-Z0-9]+'];
+
+        $this->router->whereAlphaNumeric(['1a2b3c'])->prefix('/{foo}')->group(function ($router) {
+            $router->get('/');
+        });
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
+    public function testGroupWhereInRegistrationOnRouter()
+    {
+        $wheres = ['foo' => 'one|two', 'bar' => 'one|two'];
+
+        $this->router->whereIn(['foo', 'bar'], ['one', 'two'])->prefix('/{foo}/{bar}')->group(function ($router) {
+            $router->get('/');
+        });
+
+        $this->router->whereIn(['bar', 'foo'], ['one', 'two'])->prefix('/api/{bar}/{foo}')->group(function ($router) {
+            $router->get('/');
+        });
+
+        /** @var \Illuminate\Routing\Route $route */
+        foreach ($this->router->getRoutes() as $route) {
+            $this->assertEquals($wheres, $route->wheres);
+        }
+    }
+
     public function testCanSetRouteName()
     {
         $this->router->as('users.index')->get('users', function () {
@@ -872,6 +1098,203 @@ class RouteRegistrarTest extends TestCase
         $this->assertSame('users.index', $this->getRoute()->getName());
     }
 
+    public function testPushMiddlewareToGroup()
+    {
+        $this->router->middlewareGroup('web', []);
+        $this->router->pushMiddlewareToGroup('web', 'test-middleware');
+
+        $this->assertEquals(['test-middleware'], $this->router->getMiddlewareGroups()['web']);
+    }
+
+    public function testPushMiddlewareToGroupUnregisteredGroup()
+    {
+        $this->router->pushMiddlewareToGroup('web', 'test-middleware');
+
+        $this->assertEquals(['test-middleware'], $this->router->getMiddlewareGroups()['web']);
+    }
+
+    public function testPushMiddlewareToGroupDuplicatedMiddleware()
+    {
+        $this->router->pushMiddlewareToGroup('web', 'test-middleware');
+        $this->router->pushMiddlewareToGroup('web', 'test-middleware');
+
+        $this->assertEquals(['test-middleware'], $this->router->getMiddlewareGroups()['web']);
+    }
+
+    public function testCanRemoveMiddlewareFromGroup()
+    {
+        $this->router->pushMiddlewareToGroup('web', 'test-middleware');
+
+        $this->router->removeMiddlewareFromGroup('web', 'test-middleware');
+
+        $this->assertEquals([], $this->router->getMiddlewareGroups()['web']);
+    }
+
+    public function testCanRemoveMiddlewareFromGroupNotUnregisteredMiddleware()
+    {
+        $this->router->middlewareGroup('web', []);
+
+        $this->router->removeMiddlewareFromGroup('web', 'different-test-middleware');
+
+        $this->assertEquals([], $this->router->getMiddlewareGroups()['web']);
+    }
+
+    public function testCanRemoveMiddlewareFromGroupUnregisteredGroup()
+    {
+        $this->router->removeMiddlewareFromGroup('web', ['test-middleware']);
+
+        $this->assertEquals([], $this->router->getMiddlewareGroups());
+    }
+
+    public function testCanRegisterSingleton()
+    {
+        $this->router->singleton('user', RouteRegistrarControllerStub::class);
+
+        $this->assertCount(3, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.show'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.edit'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.update'));
+    }
+
+    public function testCanRegisterApiSingleton()
+    {
+        $this->router->apiSingleton('user', RouteRegistrarControllerStub::class);
+
+        $this->assertCount(2, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.show'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.update'));
+    }
+
+    public function testCanRegisterCreatableSingleton()
+    {
+        $this->router->singleton('user', RouteRegistrarControllerStub::class)->creatable();
+
+        $this->assertCount(6, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.create'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.store'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.show'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.edit'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.update'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.destroy'));
+    }
+
+    public function testCanRegisterCreatableApiSingleton()
+    {
+        $this->router->apiSingleton('user', RouteRegistrarControllerStub::class)->creatable();
+
+        $this->assertCount(4, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.store'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.show'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.update'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.destroy'));
+    }
+
+    public function testSingletonCreatableNotDestroyable()
+    {
+        $this->router->singleton('user', RouteRegistrarControllerStub::class)
+            ->creatable()
+            ->except('destroy');
+
+        $this->assertCount(5, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.create'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.store'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.show'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.edit'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.update'));
+        $this->assertFalse($this->router->getRoutes()->hasNamedRoute('user.destroy'));
+    }
+
+    public function testApiSingletonCreatableNotDestroyable()
+    {
+        $this->router->apiSingleton('user', RouteRegistrarControllerStub::class)
+            ->creatable()
+            ->except('destroy');
+
+        $this->assertCount(3, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.store'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.show'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.update'));
+        $this->assertFalse($this->router->getRoutes()->hasNamedRoute('user.destroy'));
+    }
+
+    public function testSingletonCanBeDestroyable()
+    {
+        $this->router->singleton('user', RouteRegistrarControllerStub::class)
+            ->destroyable();
+
+        $this->assertCount(4, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.show'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.edit'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.update'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.destroy'));
+    }
+
+    public function testApiSingletonCanBeDestroyable()
+    {
+        $this->router->apiSingleton('user', RouteRegistrarControllerStub::class)
+            ->destroyable();
+
+        $this->assertCount(3, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.show'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.update'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.destroy'));
+    }
+
+    public function testSingletonCanBeOnlyCreatable()
+    {
+        $this->router->singleton('user', RouteRegistrarControllerStub::class)
+            ->creatable()
+            ->only('create', 'store');
+
+        $this->assertCount(2, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.create'));
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.store'));
+    }
+
+    public function testApiSingletonCanBeOnlyCreatable()
+    {
+        $this->router->apiSingleton('user', RouteRegistrarControllerStub::class)
+            ->creatable()
+            ->only('store');
+
+        $this->assertCount(1, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.store'));
+    }
+
+    public function testSingletonDoesntAllowIncludingUnsupportedMethods()
+    {
+        $this->router->singleton('post', RouteRegistrarControllerStub::class)
+            ->only('index', 'store', 'create', 'destroy');
+
+        $this->assertCount(0, $this->router->getRoutes());
+
+        $this->router->apiSingleton('user', RouteRegistrarControllerStub::class)
+            ->only('index', 'store', 'create', 'destroy');
+
+        $this->assertCount(0, $this->router->getRoutes());
+    }
+
+    public function testApiSingletonCanIncludeAnySingletonMethods()
+    {
+        // This matches the behavior of the apiResource method.
+        $this->router->apiSingleton('user', RouteRegistrarControllerStub::class)
+            ->only('edit');
+
+        $this->assertCount(1, $this->router->getRoutes());
+
+        $this->assertTrue($this->router->getRoutes()->hasNamedRoute('user.edit'));
+    }
+
     /**
      * Get the last route registered with the router.
      *
