@@ -14,6 +14,8 @@ use PHPUnit\Framework\TestCase;
 use Symfony\Component\HttpFoundation\Request as SymfonyRequest;
 use Symfony\Component\Routing\Exception\RouteNotFoundException;
 
+include_once 'Enums.php';
+
 class RoutingUrlGeneratorTest extends TestCase
 {
     public function testBasicGeneration()
@@ -37,10 +39,10 @@ class RoutingUrlGeneratorTest extends TestCase
         );
 
         $this->assertSame('https://www.foo.com/foo/bar', $url->to('foo/bar'));
+    }
 
-        /*
-         * Test asset URL generation...
-         */
+    public function testAssetGeneration()
+    {
         $url = new UrlGenerator(
             new RouteCollection,
             Request::create('http://www.foo.com/index.php/')
@@ -48,6 +50,15 @@ class RoutingUrlGeneratorTest extends TestCase
 
         $this->assertSame('http://www.foo.com/foo/bar', $url->asset('foo/bar'));
         $this->assertSame('https://www.foo.com/foo/bar', $url->asset('foo/bar', true));
+
+        $url = new UrlGenerator(
+            new RouteCollection,
+            Request::create('http://www.foo.com/index.php/'),
+            '/'
+        );
+
+        $this->assertSame('/foo/bar', $url->asset('foo/bar'));
+        $this->assertSame('/foo/bar', $url->asset('foo/bar', true));
     }
 
     public function testBasicGenerationWithHostFormatting()
@@ -373,6 +384,51 @@ class RoutingUrlGeneratorTest extends TestCase
         $this->assertSame('/foo/test-slug', $url->route('routable', [$model], false));
     }
 
+    public function testRoutableInterfaceRoutingAsQueryString()
+    {
+        $url = new UrlGenerator(
+            $routes = new RouteCollection,
+            Request::create('http://www.foo.com/')
+        );
+
+        $route = new Route(['GET'], 'foo', ['as' => 'query-string']);
+        $routes->add($route);
+
+        $model = new RoutableInterfaceStub;
+        $model->key = 'routable';
+
+        $this->assertSame('/foo?routable', $url->route('query-string', $model, false));
+        $this->assertSame('/foo?routable', $url->route('query-string', [$model], false));
+        $this->assertSame('/foo?foo=routable', $url->route('query-string', ['foo' => $model], false));
+    }
+
+    /**
+     * @todo Fix bug related to route keys
+     *
+     * @link https://github.com/laravel/framework/pull/42425
+     */
+    public function testRoutableInterfaceRoutingWithSeparateBindingFieldOnlyForSecondParameter()
+    {
+        $this->markTestSkipped('See https://github.com/laravel/framework/pull/43255');
+
+        $url = new UrlGenerator(
+            $routes = new RouteCollection,
+            Request::create('http://www.foo.com/')
+        );
+
+        $route = new Route(['GET'], 'foo/{bar}/{baz:slug}', ['as' => 'routable']);
+        $routes->add($route);
+
+        $model1 = new RoutableInterfaceStub;
+        $model1->key = 'routable-1';
+
+        $model2 = new RoutableInterfaceStub;
+        $model2->key = 'routable-2';
+
+        $this->assertSame('/foo/routable-1/test-slug', $url->route('routable', ['bar' => $model1, 'baz' => $model2], false));
+        $this->assertSame('/foo/routable-1/test-slug', $url->route('routable', [$model1, $model2], false));
+    }
+
     public function testRoutableInterfaceRoutingWithSingleParameter()
     {
         $url = new UrlGenerator(
@@ -509,7 +565,7 @@ class RoutingUrlGeneratorTest extends TestCase
 
     public function testRoutesWithDomainsThroughProxy()
     {
-        Request::setTrustedProxies(['10.0.0.1'], SymfonyRequest::HEADER_X_FORWARDED_ALL);
+        Request::setTrustedProxies(['10.0.0.1'], SymfonyRequest::HEADER_X_FORWARDED_FOR | SymfonyRequest::HEADER_X_FORWARDED_HOST | SymfonyRequest::HEADER_X_FORWARDED_PORT | SymfonyRequest::HEADER_X_FORWARDED_PROTO);
 
         $url = new UrlGenerator(
             $routes = new RouteCollection,
@@ -522,7 +578,7 @@ class RoutingUrlGeneratorTest extends TestCase
         $this->assertSame('http://sub.foo.com/foo/bar', $url->route('foo'));
     }
 
-    public function providerRouteParameters()
+    public static function providerRouteParameters()
     {
         return [
             [['test' => 123]],
@@ -551,7 +607,7 @@ class RoutingUrlGeneratorTest extends TestCase
         $this->assertSame('http://www.foo.com:8080/foo?test=123', $url->route('foo', $parameters));
     }
 
-    public function provideParametersAndExpectedMeaningfulExceptionMessages()
+    public static function provideParametersAndExpectedMeaningfulExceptionMessages()
     {
         return [
             'Missing parameters "one", "two" and "three"' => [
@@ -654,6 +710,28 @@ class RoutingUrlGeneratorTest extends TestCase
         $this->assertEquals($url->to('/foo'), $url->previous('/foo'));
     }
 
+    public function testPreviousPath()
+    {
+        $url = new UrlGenerator(
+            new RouteCollection,
+            Request::create('http://www.foo.com/')
+        );
+
+        $url->getRequest()->headers->set('referer', 'http://www.foo.com?baz=bah');
+        $this->assertSame('/', $url->previousPath());
+
+        $url->getRequest()->headers->set('referer', 'http://www.foo.com/?baz=bah');
+        $this->assertSame('/', $url->previousPath());
+
+        $url->getRequest()->headers->set('referer', 'http://www.foo.com/bar?baz=bah');
+        $this->assertSame('/bar', $url->previousPath());
+
+        $url->getRequest()->headers->remove('referer');
+        $this->assertSame('/', $url->previousPath());
+
+        $this->assertSame('/bar', $url->previousPath('/bar'));
+    }
+
     public function testRouteNotDefinedException()
     {
         $this->expectException(RouteNotFoundException::class);
@@ -686,7 +764,7 @@ class RoutingUrlGeneratorTest extends TestCase
 
         $this->assertTrue($url->hasValidSignature($request));
 
-        $request = Request::create($url->signedRoute('foo').'?tempered=true');
+        $request = Request::create($url->signedRoute('foo').'?tampered=true');
 
         $this->assertFalse($url->hasValidSignature($request));
     }
@@ -734,7 +812,7 @@ class RoutingUrlGeneratorTest extends TestCase
 
         $this->assertTrue($url->hasValidSignature($request, false));
 
-        $request = Request::create($url->signedRoute('foo', [], null, false).'?tempered=true');
+        $request = Request::create($url->signedRoute('foo', [], null, false).'?tampered=true');
 
         $this->assertFalse($url->hasValidSignature($request, false));
     }
@@ -780,6 +858,66 @@ class RoutingUrlGeneratorTest extends TestCase
 
         Request::create($url->signedRoute('foo', ['expires' => 253402300799]));
     }
+
+    public function testRouteGenerationWithBackedEnums()
+    {
+        $url = new UrlGenerator(
+            $routes = new RouteCollection,
+            Request::create('http://www.foo.com/')
+        );
+
+        $namedRoute = new Route(['GET'], '/foo/{bar}', ['as' => 'foo.bar']);
+        $routes->add($namedRoute);
+
+        $this->assertSame('http://www.foo.com/foo/fruits', $url->route('foo.bar', CategoryBackedEnum::Fruits));
+    }
+
+    public function testSignedUrlWithKeyResolver()
+    {
+        $url = new UrlGenerator(
+            $routes = new RouteCollection,
+            $request = Request::create('http://www.foo.com/')
+        );
+        $url->setKeyResolver(function () {
+            return 'secret';
+        });
+
+        $route = new Route(['GET'], 'foo', ['as' => 'foo', function () {
+            //
+        }]);
+        $routes->add($route);
+
+        $request = Request::create($url->signedRoute('foo'));
+
+        $this->assertTrue($url->hasValidSignature($request));
+
+        $request = Request::create($url->signedRoute('foo').'?tampered=true');
+
+        $this->assertFalse($url->hasValidSignature($request));
+
+        $url2 = $url->withKeyResolver(function () {
+            return 'other-secret';
+        });
+
+        $this->assertFalse($url2->hasValidSignature($request));
+
+        $request = Request::create($url2->signedRoute('foo'));
+
+        $this->assertTrue($url2->hasValidSignature($request));
+        $this->assertFalse($url->hasValidSignature($request));
+    }
+
+    public function testMissingNamedRouteResolution()
+    {
+        $url = new UrlGenerator(
+            new RouteCollection,
+            Request::create('http://www.foo.com/')
+        );
+
+        $url->resolveMissingNamedRoutesUsing(fn ($name, $parameters, $absolute) => 'test-url');
+
+        $this->assertSame('test-url', $url->route('foo'));
+    }
 }
 
 class RoutableInterfaceStub implements UrlRoutable
