@@ -45,7 +45,7 @@ class ConfigurationUrlParserTest extends TestCase
         ], (new ConfigurationUrlParser)->parseConfiguration('some-particular-alias://null'));
     }
 
-    public function databaseUrls()
+    public static function databaseUrls()
     {
         return [
             'simple URL' => [
@@ -135,7 +135,7 @@ class ConfigurationUrlParserTest extends TestCase
                 ],
             ],
             'query params from URL are used as extra params' => [
-                'url' => 'mysql://foo:bar@localhost/database?charset=UTF-8',
+                'mysql://foo:bar@localhost/database?charset=UTF-8',
                 [
                     'driver' => 'mysql',
                     'database' => 'database',
