@@ -48,7 +48,21 @@ class SupportComposerTest extends TestCase
         $composer->dumpOptimized();
     }
 
-    private function mockComposer(array $expectedProcessArguments, $customComposerPhar = false)
+    public function testRequirePackagesRunsTheCorrectCommand()
+    {
+        $composer = $this->mockComposer(['composer', 'require', 'pestphp/pest:^2.0', 'pestphp/pest-plugin-laravel:^2.0', '--dev']);
+
+        $composer->requirePackages(['pestphp/pest:^2.0', 'pestphp/pest-plugin-laravel:^2.0'], true);
+    }
+
+    public function testRemovePackagesRunsTheCorrectCommand()
+    {
+        $composer = $this->mockComposer(['composer', 'remove', 'phpunit/phpunit', '--dev']);
+
+        $composer->removePackages(['phpunit/phpunit'], true);
+    }
+
+    private function mockComposer(array $expectedProcessArguments, $customComposerPhar = false, array $environmentVariables = [])
     {
         $directory = __DIR__;
 
@@ -60,7 +74,7 @@ class SupportComposerTest extends TestCase
 
         $composer = $this->getMockBuilder(Composer::class)
             ->onlyMethods(['getProcess'])
-            ->setConstructorArgs([$files, $directory])
+            ->setConstructorArgs([$files, $directory, $environmentVariables])
             ->getMock();
         $composer->expects($this->once())
             ->method('getProcess')
