@@ -17,7 +17,6 @@ class SupportFluentTest extends TestCase
 
         $refl = new ReflectionObject($fluent);
         $attributes = $refl->getProperty('attributes');
-        $attributes->setAccessible(true);
 
         $this->assertEquals($array, $attributes->getValue($fluent));
         $this->assertEquals($array, $fluent->getAttributes());
@@ -30,7 +29,6 @@ class SupportFluentTest extends TestCase
 
         $refl = new ReflectionObject($fluent);
         $attributes = $refl->getProperty('attributes');
-        $attributes->setAccessible(true);
 
         $this->assertEquals($array, $attributes->getValue($fluent));
         $this->assertEquals($array, $fluent->getAttributes());
@@ -43,7 +41,6 @@ class SupportFluentTest extends TestCase
 
         $refl = new ReflectionObject($fluent);
         $attributes = $refl->getProperty('attributes');
-        $attributes->setAccessible(true);
 
         $this->assertEquals($array, $attributes->getValue($fluent));
         $this->assertEquals($array, $fluent->getAttributes());
@@ -108,10 +105,10 @@ class SupportFluentTest extends TestCase
     public function testToJsonEncodesTheToArrayResult()
     {
         $fluent = $this->getMockBuilder(Fluent::class)->onlyMethods(['toArray'])->getMock();
-        $fluent->expects($this->once())->method('toArray')->willReturn('foo');
+        $fluent->expects($this->once())->method('toArray')->willReturn(['foo']);
         $results = $fluent->toJson();
 
-        $this->assertJsonStringEqualsJsonString(json_encode('foo'), $results);
+        $this->assertJsonStringEqualsJsonString(json_encode(['foo']), $results);
     }
 }
 
@@ -124,8 +121,7 @@ class FluentArrayIteratorStub implements IteratorAggregate
         $this->items = $items;
     }
 
-    #[\ReturnTypeWillChange]
-    public function getIterator()
+    public function getIterator(): ArrayIterator
     {
         return new ArrayIterator($this->items);
     }
