@@ -2,9 +2,10 @@
 
 namespace Illuminate\Tests\Support;
 
-use Carbon\Carbon;
+use Illuminate\Support\Carbon;
 use Illuminate\Support\Collection;
 use Illuminate\Support\LazyCollection;
+use InvalidArgumentException;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
 
@@ -46,7 +47,7 @@ class SupportLazyCollectionTest extends TestCase
         $this->assertSame($array, $data->all());
     }
 
-    public function testCanCreateCollectionFromClosure()
+    public function testCanCreateCollectionFromGeneratorFunction()
     {
         $data = LazyCollection::make(function () {
             yield 1;
@@ -69,6 +70,26 @@ class SupportLazyCollectionTest extends TestCase
         ], $data->all());
     }
 
+    public function testCanCreateCollectionFromNonGeneratorFunction()
+    {
+        $data = LazyCollection::make(function () {
+            return 'laravel';
+        });
+
+        $this->assertSame(['laravel'], $data->all());
+    }
+
+    public function testDoesNotCreateCollectionFromGenerator()
+    {
+        $this->expectException(InvalidArgumentException::class);
+
+        $generateNumber = function () {
+            yield 1;
+        };
+
+        LazyCollection::make($generateNumber());
+    }
+
     public function testEager()
     {
         $source = [1, 2, 3, 4, 5];
@@ -164,7 +185,7 @@ class SupportLazyCollectionTest extends TestCase
 
         $results = $mock
             ->times(10)
-            ->pipe(function ($collection) use ($mock, $timeout) {
+            ->tap(function ($collection) use ($mock, $timeout) {
                 tap($collection)
                     ->mockery_init($mock->mockery_getContainer())
                     ->shouldAllowMockingProtectedMethods()
@@ -175,8 +196,6 @@ class SupportLazyCollectionTest extends TestCase
                         (clone $timeout)->sub(1, 'minute')->getTimestamp(),
                         $timeout->getTimestamp()
                     );
-
-                return $collection;
             })
             ->takeUntilTimeout($timeout)
             ->all();
