@@ -5,25 +5,32 @@ namespace Illuminate\Tests\Support;
 use Illuminate\Contracts\Events\Dispatcher;
 use Illuminate\Support\Testing\Fakes\EventFake;
 use Mockery as m;
-use PHPUnit\Framework\Constraint\ExceptionMessage;
 use PHPUnit\Framework\ExpectationFailedException;
 use PHPUnit\Framework\TestCase;
 
 class SupportTestingEventFakeTest extends TestCase
 {
+    protected $fake;
+
     protected function setUp(): void
     {
         parent::setUp();
         $this->fake = new EventFake(m::mock(Dispatcher::class));
     }
 
+    protected function tearDown(): void
+    {
+        parent::tearDown();
+        m::close();
+    }
+
     public function testAssertDispatched()
     {
         try {
             $this->fake->assertDispatched(EventStub::class);
             $this->fail();
         } catch (ExpectationFailedException $e) {
-            $this->assertThat($e, new ExceptionMessage('The expected [Illuminate\Tests\Support\EventStub] event was not dispatched.'));
+            $this->assertStringContainsString('The expected [Illuminate\Tests\Support\EventStub] event was not dispatched.', $e->getMessage());
         }
 
         $this->fake->dispatch(EventStub::class);
@@ -40,6 +47,20 @@ class SupportTestingEventFakeTest extends TestCase
         });
     }
 
+    public function testAssertListening()
+    {
+        $listener = ListenerStub::class;
+
+        $dispatcher = m::mock(Dispatcher::class);
+        $dispatcher->shouldReceive('getListeners')->andReturn([function ($event, $payload) use ($listener) {
+            return $listener(...array_values($payload));
+        }]);
+
+        $fake = new EventFake($dispatcher);
+
+        $fake->assertListening(EventStub::class, ListenerStub::class);
+    }
+
     public function testAssertDispatchedWithCallbackInt()
     {
         $this->fake->dispatch(EventStub::class);
@@ -49,7 +70,7 @@ class SupportTestingEventFakeTest extends TestCase
             $this->fake->assertDispatched(EventStub::class, 1);
             $this->fail();
         } catch (ExpectationFailedException $e) {
-            $this->assertThat($e, new ExceptionMessage('The expected [Illuminate\Tests\Support\EventStub] event was dispatched 2 times instead of 1 times.'));
+            $this->assertStringContainsString('The expected [Illuminate\Tests\Support\EventStub] event was dispatched 2 times instead of 1 times.', $e->getMessage());
         }
 
         $this->fake->assertDispatched(EventStub::class, 2);
@@ -64,7 +85,7 @@ class SupportTestingEventFakeTest extends TestCase
             $this->fake->assertDispatchedTimes(EventStub::class, 1);
             $this->fail();
         } catch (ExpectationFailedException $e) {
-            $this->assertThat($e, new ExceptionMessage('The expected [Illuminate\Tests\Support\EventStub] event was dispatched 2 times instead of 1 times.'));
+            $this->assertStringContainsString('The expected [Illuminate\Tests\Support\EventStub] event was dispatched 2 times instead of 1 times.', $e->getMessage());
         }
 
         $this->fake->assertDispatchedTimes(EventStub::class, 2);
@@ -80,7 +101,7 @@ class SupportTestingEventFakeTest extends TestCase
             $this->fake->assertNotDispatched(EventStub::class);
             $this->fail();
         } catch (ExpectationFailedException $e) {
-            $this->assertThat($e, new ExceptionMessage('The unexpected [Illuminate\Tests\Support\EventStub] event was dispatched.'));
+            $this->assertStringContainsString('The unexpected [Illuminate\Tests\Support\EventStub] event was dispatched.', $e->getMessage());
         }
     }
 
@@ -94,7 +115,7 @@ class SupportTestingEventFakeTest extends TestCase
             });
             $this->fail();
         } catch (ExpectationFailedException $e) {
-            $this->assertThat($e, new ExceptionMessage('The unexpected [Illuminate\Tests\Support\EventStub] event was dispatched.'));
+            $this->assertStringContainsString('The unexpected [Illuminate\Tests\Support\EventStub] event was dispatched.', $e->getMessage());
         }
     }
 
@@ -130,7 +151,7 @@ class SupportTestingEventFakeTest extends TestCase
             $this->fake->assertNothingDispatched();
             $this->fail();
         } catch (ExpectationFailedException $e) {
-            $this->assertThat($e, new ExceptionMessage('2 unexpected events were dispatched.'));
+            $this->assertStringContainsString('2 unexpected events were dispatched.', $e->getMessage());
         }
     }
 }
@@ -139,3 +160,8 @@ class EventStub
 {
     //
 }
+
+class ListenerStub
+{
+    //
+}
