@@ -94,6 +94,12 @@ class ValidationExistsRuleTest extends TestCase
         $this->assertFalse($v->passes());
         $v->setData(['id' => 4]);
         $this->assertFalse($v->passes());
+
+        // array values
+        $v->setData(['id' => [1, 2]]);
+        $this->assertTrue($v->passes());
+        $v->setData(['id' => [3, 4]]);
+        $this->assertFalse($v->passes());
     }
 
     public function testItChoosesValidRecordsUsingWhereNotInRule()
@@ -118,6 +124,12 @@ class ValidationExistsRuleTest extends TestCase
         $this->assertTrue($v->passes());
         $v->setData(['id' => 4]);
         $this->assertTrue($v->passes());
+
+        // array values
+        $v->setData(['id' => [1, 2]]);
+        $this->assertFalse($v->passes());
+        $v->setData(['id' => [3, 4]]);
+        $this->assertTrue($v->passes());
     }
 
     public function testItChoosesValidRecordsUsingConditionalModifiers()
@@ -147,6 +159,12 @@ class ValidationExistsRuleTest extends TestCase
         $this->assertTrue($v->passes());
         $v->setData(['id' => 4]);
         $this->assertTrue($v->passes());
+
+        // array values
+        $v->setData(['id' => [1, 2]]);
+        $this->assertFalse($v->passes());
+        $v->setData(['id' => [3, 4]]);
+        $this->assertTrue($v->passes());
     }
 
     public function testItChoosesValidRecordsUsingWhereNotInAndWhereNotInRulesTogether()
@@ -158,6 +176,7 @@ class ValidationExistsRuleTest extends TestCase
         User::create(['id' => '2', 'type' => 'bar']);
         User::create(['id' => '3', 'type' => 'baz']);
         User::create(['id' => '4', 'type' => 'other']);
+        User::create(['id' => '5', 'type' => 'baz']);
 
         $trans = $this->getIlluminateArrayTranslator();
         $v = new Validator($trans, [], ['id' => $rule]);
@@ -171,6 +190,37 @@ class ValidationExistsRuleTest extends TestCase
         $this->assertTrue($v->passes());
         $v->setData(['id' => 4]);
         $this->assertFalse($v->passes());
+        $v->setData(['id' => 5]);
+        $this->assertTrue($v->passes());
+
+        // array values
+        $v->setData(['id' => [1, 2, 4]]);
+        $this->assertFalse($v->passes());
+        $v->setData(['id' => [3, 5]]);
+        $this->assertTrue($v->passes());
+    }
+
+    public function testItChoosesValidRecordsUsingWhereNotRule()
+    {
+        $rule = new Exists('users', 'id');
+
+        $rule->whereNot('type', 'baz');
+
+        User::create(['id' => '1', 'type' => 'foo']);
+        User::create(['id' => '2', 'type' => 'bar']);
+        User::create(['id' => '3', 'type' => 'baz']);
+        User::create(['id' => '4', 'type' => 'other']);
+        User::create(['id' => '5', 'type' => 'baz']);
+
+        $trans = $this->getIlluminateArrayTranslator();
+        $v = new Validator($trans, [], ['id' => $rule]);
+        $v->setPresenceVerifier(new DatabasePresenceVerifier(Eloquent::getConnectionResolver()));
+
+        $v->setData(['id' => 3]);
+        $this->assertFalse($v->passes());
+
+        $v->setData(['id' => 4]);
+        $this->assertTrue($v->passes());
     }
 
     public function testItIgnoresSoftDeletes()
@@ -184,6 +234,17 @@ class ValidationExistsRuleTest extends TestCase
         $this->assertSame('exists:table,NULL,softdeleted_at,"NULL"', (string) $rule);
     }
 
+    public function testItOnlyTrashedSoftDeletes()
+    {
+        $rule = new Exists('table');
+        $rule->onlyTrashed();
+        $this->assertSame('exists:table,NULL,deleted_at,"NOT_NULL"', (string) $rule);
+
+        $rule = new Exists('table');
+        $rule->onlyTrashed('softdeleted_at');
+        $this->assertSame('exists:table,NULL,softdeleted_at,"NOT_NULL"', (string) $rule);
+    }
+
     protected function createSchema()
     {
         $this->schema('default')->create('users', function ($table) {
