@@ -2,6 +2,8 @@
 
 namespace Illuminate\Tests\View\Blade;
 
+use Illuminate\Contracts\View\ViewCompilationException;
+
 class BladeForeachStatementsTest extends AbstractBladeTestCase
 {
     public function testForeachStatementsAreCompiled()
@@ -91,4 +93,30 @@ tag info
 
         $this->assertEquals($expected, $this->compiler->compileString($string));
     }
+
+    /**
+     * @dataProvider invalidForeachStatementsDataProvider
+     */
+    public function testForeachStatementsThrowHumanizedMessageWhenInvalidStatement($initialStatement)
+    {
+        $this->expectException(ViewCompilationException::class);
+        $this->expectExceptionMessage('Malformed @foreach statement.');
+        $string = "$initialStatement
+test
+@endforeach";
+        $this->compiler->compileString($string);
+    }
+
+    public static function invalidForeachStatementsDataProvider()
+    {
+        return [
+            ['@foreach'],
+            ['@foreach()'],
+            ['@foreach ()'],
+            ['@foreach($test)'],
+            ['@foreach($test as)'],
+            ['@foreach(as)'],
+            ['@foreach ( as )'],
+        ];
+    }
 }
