@@ -9,5 +9,9 @@ class BladeStackTest extends AbstractBladeTestCase
         $string = '@stack(\'foo\')';
         $expected = '<?php echo $__env->yieldPushContent(\'foo\'); ?>';
         $this->assertEquals($expected, $this->compiler->compileString($string));
+
+        $string = '@stack(\'foo))\')';
+        $expected = '<?php echo $__env->yieldPushContent(\'foo))\'); ?>';
+        $this->assertEquals($expected, $this->compiler->compileString($string));
     }
 }
